@@ -39,16 +39,22 @@
 /// display.
 /// @author  Yue Li and Matthew Hielsberg
 
-#ifndef CLOUD_H_
-#define CLOUD_H_
+#pragma once
 
 #include <QtGui/QColor>
 #include <pcl/apps/point_cloud_editor/localTypes.h>
 #include <pcl/apps/point_cloud_editor/statistics.h>
+
+#include <pcl/memory.h>  // for pcl::weak_ptr
+
 #ifdef OPENGL_IS_A_FRAMEWORK
 # include <OpenGL/gl.h>
 # include <OpenGL/glu.h>
 #else
+#ifdef _WIN32
+// Need this to pull in APIENTRY, etc.
+#include "windows.h"
+#endif
 # include <GL/gl.h>
 # include <GL/glu.h>
 #endif
@@ -67,11 +73,17 @@
 /// the origin.
 // XXX - handle shifting upon setting of a Cloud3D
 // XXX - add functions for retrieving an unshifted Cloud3D
-// XXX - add functions for retrieveing unshifted points by index
+// XXX - add functions for retrieving unshifted points by index
 // XXX - mark access functions below as returning shifted values
 class Cloud : public Statistics
 {
   public:
+    /// The type for shared pointer pointing to a selection buffer
+    using SelectionPtr = pcl::shared_ptr<Selection>;
+
+    /// The type for weak pointer pointing to a selection buffer
+    using SelectionWeakPtr = pcl::weak_ptr<Selection>;
+
     /// @brief Default Constructor
     Cloud ();
 
@@ -87,9 +99,6 @@ class Cloud : public Statistics
     /// variables of this object are initialized but not set.
     Cloud (const Cloud3D& cloud, bool register_stats=false);
 
-    /// @brief Destructor
-    ~Cloud ();
-
     /// @brief Equal Operator
     /// @details Deep copies all the state of the passed cloud to this cloud.
     /// @param cloud The cloud object whose status to be copied to this object
@@ -201,7 +210,7 @@ class Cloud : public Statistics
     /// a faster rendering mode; this also occurs if the selection object is
     /// empty.
     void
-    setSelection (SelectionPtr selection_ptr);
+    setSelection (const SelectionPtr& selection_ptr);
 
     /// @brief Sets the RGB values for coloring points in COLOR_BY_PURE mode.
     /// @param r the value for red color
@@ -247,7 +256,7 @@ class Cloud : public Statistics
     void
     drawWithHighlightColor () const;
 
-    /// @brief Sets the axis along which the displyed points should have the
+    /// @brief Sets the axis along which the displayed points should have the
     /// color ramp applied.
     /// @param a The axis id describing which direction the ramp should be
     /// applied.
@@ -369,23 +378,23 @@ class Cloud : public Statistics
 
     /// @brief Get statistics of the selected points in string.
     std::string
-    getStat () const;
+    getStat () const override;
 
     /// Default Point Size
     static const float DEFAULT_POINT_DISPLAY_SIZE_;
     /// Default Highlight Point Size
     static const float DEFAULT_POINT_HIGHLIGHT_SIZE_;
-    /// Default Point Color - Red componenet
+    /// Default Point Color - Red component
     static const float DEFAULT_POINT_DISPLAY_COLOR_RED_;
-    /// Default Point Color - Green componenet
+    /// Default Point Color - Green component
     static const float DEFAULT_POINT_DISPLAY_COLOR_GREEN_;
-    /// Default Point Color - Blue componenet
+    /// Default Point Color - Blue component
     static const float DEFAULT_POINT_DISPLAY_COLOR_BLUE_;
-    /// Default Point Highlight Color - Red componenet
+    /// Default Point Highlight Color - Red component
     static const float DEFAULT_POINT_HIGHLIGHT_COLOR_RED_;
-    /// Default Point Highlight Color - Green componenet
+    /// Default Point Highlight Color - Green component
     static const float DEFAULT_POINT_HIGHLIGHT_COLOR_GREEN_;
-    /// Default Point Highlight Color - Blue componenet
+    /// Default Point Highlight Color - Blue component
     static const float DEFAULT_POINT_HIGHLIGHT_COLOR_BLUE_;
 
   private:
@@ -409,19 +418,15 @@ class Cloud : public Statistics
     /// The internal representation of the cloud
     Cloud3D cloud_;
 
-    /// @breif A weak pointer pointing to the selection object.
+    /// @brief A weak pointer pointing to the selection object.
     /// @details This implementation uses the weak pointer to allow for a lazy
     /// update of the cloud if the selection object is destroyed.
-    boost::weak_ptr<Selection> selection_wk_ptr_;
+    SelectionWeakPtr selection_wk_ptr_;
 
     /// Flag that indicates whether a color ramp should be used (true) or not
     /// (false) when displaying the cloud
     bool use_color_ramp_;
 
-    /// Flag that indicates whether the cloud should be colored with its own
-    /// color
-    bool use_native_color_;
-
     /// The axis which the color ramp is to be applied when drawing the cloud
     Axis color_ramp_axis_;
 
@@ -472,8 +477,3 @@ class Cloud : public Statistics
     /// The translations on x, y, and z axis on the selected points.
     float select_translate_x_, select_translate_y_, select_translate_z_;
 };
-#endif // CLOUD_H_
-
-
-
-
