@@ -37,8 +37,7 @@
  *
  */
 
-#ifndef PCL_CONVERSIONS_H_
-#define PCL_CONVERSIONS_H_
+#pragma once
 
 #ifdef __GNUC__
 #pragma GCC system_header
@@ -48,13 +47,13 @@
 #include <pcl/PCLPointCloud2.h>
 #include <pcl/PCLImage.h>
 #include <pcl/point_cloud.h>
-#include <pcl/point_traits.h>
+#include <pcl/type_traits.h>
 #include <pcl/for_each_type.h>
-#include <pcl/exceptions.h>
 #include <pcl/console/print.h>
-#ifndef Q_MOC_RUN
-#include <boost/foreach.hpp>
-#endif
+
+#include <algorithm>
+#include <iterator>
+#include <numeric> // for accumulate
 
 namespace pcl
 {
@@ -69,10 +68,10 @@ namespace pcl
       template<typename U> void operator() ()
       {
         pcl::PCLPointField f;
-        f.name = traits::name<PointT, U>::value;
-        f.offset = traits::offset<PointT, U>::value;
-        f.datatype = traits::datatype<PointT, U>::value;
-        f.count = traits::datatype<PointT, U>::size;
+        f.name = pcl::traits::name<PointT, U>::value;
+        f.offset = pcl::traits::offset<PointT, U>::value;
+        f.datatype = pcl::traits::datatype<PointT, U>::value;
+        f.count = pcl::traits::datatype<PointT, U>::size;
         fields_.push_back (f);
       }
 
@@ -92,20 +91,20 @@ namespace pcl
       template<typename Tag> void
       operator () ()
       {
-        BOOST_FOREACH (const pcl::PCLPointField& field, fields_)
+        for (const auto& field : fields_)
         {
           if (FieldMatches<PointT, Tag>()(field))
           {
             FieldMapping mapping;
             mapping.serialized_offset = field.offset;
-            mapping.struct_offset = traits::offset<PointT, Tag>::value;
-            mapping.size = sizeof (typename traits::datatype<PointT, Tag>::type);
+            mapping.struct_offset = pcl::traits::offset<PointT, Tag>::value;
+            mapping.size = sizeof (typename pcl::traits::datatype<PointT, Tag>::type);
             map_.push_back (mapping);
             return;
           }
         }
         // Disable thrown exception per #595: http://dev.pointclouds.org/issues/595
-        PCL_WARN ("Failed to find match for field '%s'.\n", traits::name<PointT, Tag>::value);
+        PCL_WARN ("Failed to find match for field '%s'.\n", pcl::traits::name<PointT, Tag>::value);
         //throw pcl::InvalidConversionException (ss.str ());
       }
 
@@ -152,6 +151,80 @@ namespace pcl
     }
   }
 
+  /** \brief Convert a PCLPointCloud2 binary data blob into a pcl::PointCloud<T> object using a field_map.
+    * \param[in] msg the PCLPointCloud2 binary blob (note that the binary point data in msg.data will not be used!)
+    * \param[out] cloud the resultant pcl::PointCloud<T>
+    * \param[in] field_map a MsgFieldMap object
+    * \param[in] msg_data pointer to binary blob data, used instead of msg.data
+    *
+    * \note Use fromPCLPointCloud2 (PCLPointCloud2, PointCloud<T>) instead, except if you have a binary blob of
+    * point data that you do not want to copy into a pcl::PCLPointCloud2 in order to use fromPCLPointCloud2.
+    */
+  template <typename PointT> void
+  fromPCLPointCloud2 (const pcl::PCLPointCloud2& msg, pcl::PointCloud<PointT>& cloud,
+              const MsgFieldMap& field_map, const std::uint8_t* msg_data)
+  {
+    // Copy info fields
+    cloud.header   = msg.header;
+    cloud.width    = msg.width;
+    cloud.height   = msg.height;
+    cloud.is_dense = msg.is_dense == 1;
+
+    // Resize cloud
+    cloud.resize (msg.width * msg.height);
+
+    // check if there is data to copy
+    if (msg.width * msg.height == 0)
+    {
+      PCL_WARN("[pcl::fromPCLPointCloud2] No data to copy.\n");
+      return;
+    }
+
+    // Copy point data
+    std::uint8_t* cloud_data = reinterpret_cast<std::uint8_t*>(cloud.data());
+
+    // Check if we can copy adjacent points in a single memcpy.  We can do so if there
+    // is exactly one field to copy and it is the same size as the source and destination
+    // point types.
+    if (field_map.size() == 1 &&
+        field_map[0].serialized_offset == 0 &&
+        field_map[0].struct_offset == 0 &&
+        field_map[0].size == msg.point_step &&
+        field_map[0].size == sizeof(PointT))
+    {
+      const auto cloud_row_step = (sizeof (PointT) * cloud.width);
+      // Should usually be able to copy all rows at once
+      if (msg.row_step == cloud_row_step)
+      {
+        memcpy (cloud_data, msg_data, msg.width * msg.height * sizeof(PointT));
+      }
+      else
+      {
+        for (uindex_t i = 0; i < msg.height; ++i, cloud_data += cloud_row_step, msg_data += msg.row_step)
+          memcpy (cloud_data, msg_data, cloud_row_step);
+      }
+
+    }
+    else
+    {
+      // If not, memcpy each group of contiguous fields separately
+      for (std::size_t row = 0; row < msg.height; ++row)
+      {
+        const std::uint8_t* row_data = msg_data + row * msg.row_step;
+        for (std::size_t col = 0; col < msg.width; ++col)
+        {
+          const std::uint8_t* msg_data = row_data + col * msg.point_step;
+          for (const detail::FieldMapping& mapping : field_map)
+          {
+            std::copy(msg_data + mapping.serialized_offset, msg_data + mapping.serialized_offset + mapping.size,
+                        cloud_data + mapping.struct_offset);
+          }
+          cloud_data += sizeof (PointT);
+        }
+      }
+    }
+  }
+
   /** \brief Convert a PCLPointCloud2 binary data blob into a pcl::PointCloud<T> object using a field_map.
     * \param[in] msg the PCLPointCloud2 binary blob
     * \param[out] cloud the resultant pcl::PointCloud<T>
@@ -169,57 +242,7 @@ namespace pcl
   fromPCLPointCloud2 (const pcl::PCLPointCloud2& msg, pcl::PointCloud<PointT>& cloud,
               const MsgFieldMap& field_map)
   {
-    // Copy info fields
-    cloud.header   = msg.header;
-    cloud.width    = msg.width;
-    cloud.height   = msg.height;
-    cloud.is_dense = msg.is_dense == 1;
-
-    // Copy point data
-    uint32_t num_points = msg.width * msg.height;
-    cloud.points.resize (num_points);
-    uint8_t* cloud_data = reinterpret_cast<uint8_t*>(&cloud.points[0]);
-
-    // Check if we can copy adjacent points in a single memcpy.  We can do so if there
-    // is exactly one field to copy and it is the same size as the source and destination
-    // point types.
-    if (field_map.size() == 1 &&
-        field_map[0].serialized_offset == 0 &&
-        field_map[0].struct_offset == 0 &&
-        field_map[0].size == msg.point_step &&
-        field_map[0].size == sizeof(PointT))
-    {
-      uint32_t cloud_row_step = static_cast<uint32_t> (sizeof (PointT) * cloud.width);
-      const uint8_t* msg_data = &msg.data[0];
-      // Should usually be able to copy all rows at once
-      if (msg.row_step == cloud_row_step)
-      {
-        memcpy (cloud_data, msg_data, msg.data.size ());
-      }
-      else
-      {
-        for (uint32_t i = 0; i < msg.height; ++i, cloud_data += cloud_row_step, msg_data += msg.row_step)
-          memcpy (cloud_data, msg_data, cloud_row_step);
-      }
-
-    }
-    else
-    {
-      // If not, memcpy each group of contiguous fields separately
-      for (uint32_t row = 0; row < msg.height; ++row)
-      {
-        const uint8_t* row_data = &msg.data[row * msg.row_step];
-        for (uint32_t col = 0; col < msg.width; ++col)
-        {
-          const uint8_t* msg_data = row_data + col * msg.point_step;
-          BOOST_FOREACH (const detail::FieldMapping& mapping, field_map)
-          {
-            memcpy (cloud_data + mapping.struct_offset, msg_data + mapping.serialized_offset, mapping.size);
-          }
-          cloud_data += sizeof (PointT);
-        }
-      }
-    }
+    fromPCLPointCloud2 (msg, cloud, field_map, msg.data.data());
   }
 
   /** \brief Convert a PCLPointCloud2 binary data blob into a pcl::PointCloud<T> object.
@@ -234,6 +257,104 @@ namespace pcl
     fromPCLPointCloud2 (msg, cloud, field_map);
   }
 
+  namespace detail {
+    /** \brief Used together with `pcl::for_each_type`, copies all point fields from `cloud_data` (respecting each field offset) to `msg_data` (tightly packed).
+      */
+    template<typename PointT>
+    struct FieldCopier {
+      FieldCopier(std::uint8_t*& msg_data, const std::uint8_t*& cloud_data) : msg_data_ (msg_data), cloud_data_ (cloud_data) {};
+
+      template<typename U> void operator() () {
+        memcpy(msg_data_, cloud_data_ + pcl::traits::offset<PointT, U>::value, sizeof(typename pcl::traits::datatype<PointT, U>::type));
+        msg_data_ += sizeof(typename pcl::traits::datatype<PointT, U>::type);
+      }
+
+      std::uint8_t*& msg_data_;
+      const std::uint8_t*& cloud_data_;
+    };
+
+    /** \brief Used together with `pcl::for_each_type`, creates list of all fields, and list of size of each field.
+      */
+    template<typename PointT>
+    struct FieldAdderAdvanced
+    {
+      FieldAdderAdvanced (std::vector<pcl::PCLPointField>& fields, std::vector<std::size_t>& field_sizes) : fields_ (fields), field_sizes_ (field_sizes) {};
+
+      template<typename U> void operator() ()
+      {
+        pcl::PCLPointField f;
+        f.name = pcl::traits::name<PointT, U>::value;
+        f.offset = pcl::traits::offset<PointT, U>::value;
+        f.datatype = pcl::traits::datatype<PointT, U>::value;
+        f.count = pcl::traits::datatype<PointT, U>::size;
+        fields_.push_back (f);
+        field_sizes_.push_back (sizeof(typename pcl::traits::datatype<PointT, U>::type)); // If field is an array, then this is the size of all array elements
+      }
+
+      std::vector<pcl::PCLPointField>& fields_;
+      std::vector<std::size_t>& field_sizes_;
+    };
+  } // namespace detail
+
+  /** \brief Convert a pcl::PointCloud<T> object to a PCLPointCloud2 binary data blob.
+    * \param[in] cloud the input pcl::PointCloud<T>
+    * \param[out] msg the resultant PCLPointCloud2 binary blob
+    * \param[in] padding Many point types have padding to ensure alignment and SIMD compatibility. Setting this to true will copy the padding to the `PCLPointCloud2` (the default in older PCL versions). Setting this to false will make the data blob in `PCLPointCloud2` smaller, while still keeping all information (useful e.g. when sending msg over network or storing it). The amount of padding depends on the point type, and can in some cases be up to 50 percent.
+    */
+  template<typename PointT> void
+  toPCLPointCloud2 (const pcl::PointCloud<PointT>& cloud, pcl::PCLPointCloud2& msg, bool padding)
+  {
+    // Ease the user's burden on specifying width/height for unorganized datasets
+    if (cloud.width == 0 && cloud.height == 0)
+    {
+      msg.width  = cloud.size ();
+      msg.height = 1;
+    }
+    else
+    {
+      assert (cloud.size () == cloud.width * cloud.height);
+      msg.height = cloud.height;
+      msg.width  = cloud.width;
+    }
+    // Fill fields metadata
+    msg.fields.clear ();
+    std::vector<std::size_t> field_sizes;
+    for_each_type<typename pcl::traits::fieldList<PointT>::type>(pcl::detail::FieldAdderAdvanced<PointT>(msg.fields, field_sizes));
+    // Check if padding should be kept, or if the point type does not contain padding (then the single memcpy is faster)
+    if (padding || std::accumulate(field_sizes.begin(), field_sizes.end(), static_cast<std::size_t>(0)) == sizeof (PointT)) {
+      // Fill point cloud binary data (padding and all)
+      std::size_t data_size = sizeof (PointT) * cloud.size ();
+      msg.data.resize (data_size);
+      if (data_size)
+      {
+        memcpy(msg.data.data(), cloud.data(), data_size);
+      }
+
+      msg.point_step = sizeof (PointT);
+      msg.row_step   = (sizeof (PointT) * msg.width);
+    } else {
+      std::size_t point_size = 0;
+      for(std::size_t i=0; i<msg.fields.size(); ++i) {
+        msg.fields[i].offset = point_size; // Adjust offset when padding is removed
+        point_size += field_sizes[i];
+      }
+      msg.data.resize (point_size * cloud.size());
+      std::uint8_t* msg_data = &msg.data[0];
+      const std::uint8_t* cloud_data=reinterpret_cast<const std::uint8_t*>(&cloud[0]);
+      const std::uint8_t* end = cloud_data + sizeof (PointT) * cloud.size ();
+      pcl::detail::FieldCopier<PointT> copier(msg_data, cloud_data); // copier takes msg_data and cloud_data as references, so the two are shared
+      for (; cloud_data<end; cloud_data+=sizeof(PointT)) {
+        for_each_type< typename traits::fieldList<PointT>::type > (copier);
+      }
+
+      msg.point_step = point_size;
+      msg.row_step   = point_size * msg.width;
+    }
+    msg.header     = cloud.header;
+    msg.is_dense   = cloud.is_dense;
+    /// @todo msg.is_bigendian = ?;
+  }
+
   /** \brief Convert a pcl::PointCloud<T> object to a PCLPointCloud2 binary data blob.
     * \param[in] cloud the input pcl::PointCloud<T>
     * \param[out] msg the resultant PCLPointCloud2 binary blob
@@ -241,36 +362,7 @@ namespace pcl
   template<typename PointT> void
   toPCLPointCloud2 (const pcl::PointCloud<PointT>& cloud, pcl::PCLPointCloud2& msg)
   {
-    // Ease the user's burden on specifying width/height for unorganized datasets
-    if (cloud.width == 0 && cloud.height == 0)
-    {
-      msg.width  = static_cast<uint32_t>(cloud.points.size ());
-      msg.height = 1;
-    }
-    else
-    {
-      assert (cloud.points.size () == cloud.width * cloud.height);
-      msg.height = cloud.height;
-      msg.width  = cloud.width;
-    }
-
-    // Fill point cloud binary data (padding and all)
-    size_t data_size = sizeof (PointT) * cloud.points.size ();
-    msg.data.resize (data_size);
-    if (data_size)
-    {
-      memcpy(&msg.data[0], &cloud.points[0], data_size);
-    }
-
-    // Fill fields metadata
-    msg.fields.clear ();
-    for_each_type<typename traits::fieldList<PointT>::type> (detail::FieldAdder<PointT>(msg.fields));
-
-    msg.header     = cloud.header;
-    msg.point_step = sizeof (PointT);
-    msg.row_step   = static_cast<uint32_t> (sizeof (PointT) * msg.width);
-    msg.is_dense   = cloud.is_dense;
-    /// @todo msg.is_bigendian = ?;
+    toPCLPointCloud2 (cloud, msg, true); // true is the default in older PCL version
   }
 
    /** \brief Copy the RGB fields of a PointCloud into pcl::PCLImage format
@@ -287,22 +379,23 @@ namespace pcl
       throw std::runtime_error("Needs to be a dense like cloud!!");
     else
     {
-      if (cloud.points.size () != cloud.width * cloud.height)
+      if (cloud.size () != cloud.width * cloud.height)
         throw std::runtime_error("The width and height do not match the cloud size!");
       msg.height = cloud.height;
       msg.width = cloud.width;
     }
 
     // ensor_msgs::image_encodings::BGR8;
+    msg.header = cloud.header;
     msg.encoding = "bgr8";
-    msg.step = msg.width * sizeof (uint8_t) * 3;
+    msg.step = msg.width * sizeof (std::uint8_t) * 3;
     msg.data.resize (msg.step * msg.height);
-    for (size_t y = 0; y < cloud.height; y++)
+    for (std::size_t y = 0; y < cloud.height; y++)
     {
-      for (size_t x = 0; x < cloud.width; x++)
+      for (std::size_t x = 0; x < cloud.width; x++)
       {
-        uint8_t * pixel = &(msg.data[y * msg.step + x * 3]);
-        memcpy (pixel, &cloud (x, y).rgb, 3 * sizeof(uint8_t));
+        std::uint8_t * pixel = &(msg.data[y * msg.step + x * 3]);
+        memcpy (pixel, &cloud (x, y).rgb, 3 * sizeof(std::uint8_t));
       }
     }
   }
@@ -315,17 +408,12 @@ namespace pcl
   inline void
   toPCLPointCloud2 (const pcl::PCLPointCloud2& cloud, pcl::PCLImage& msg)
   {
-    int rgb_index = -1;
-    // Get the index we need
-    for (size_t d = 0; d < cloud.fields.size (); ++d)
-      if (cloud.fields[d].name == "rgb")
-      {
-        rgb_index = static_cast<int>(d);
-        break;
-      }
-
-    if(rgb_index == -1)
+    const auto predicate = [](const auto& field) { return field.name == "rgb"; };
+    const auto result = std::find_if(cloud.fields.cbegin (), cloud.fields.cend (), predicate);
+    if (result == cloud.fields.end ())
       throw std::runtime_error ("No rgb field!!");
+
+    const auto rgb_index = std::distance(cloud.fields.begin (), result);
     if (cloud.width == 0 && cloud.height == 0)
       throw std::runtime_error ("Needs to be a dense like cloud!!");
     else
@@ -333,23 +421,22 @@ namespace pcl
       msg.height = cloud.height;
       msg.width = cloud.width;
     }
-    int rgb_offset = cloud.fields[rgb_index].offset;
-    int point_step = cloud.point_step;
+    auto rgb_offset = cloud.fields[rgb_index].offset;
+    const auto point_step = cloud.point_step;
 
     // pcl::image_encodings::BGR8;
+    msg.header = cloud.header;
     msg.encoding = "bgr8";
-    msg.step = static_cast<uint32_t>(msg.width * sizeof (uint8_t) * 3);
+    msg.step = (msg.width * sizeof (std::uint8_t) * 3);
     msg.data.resize (msg.step * msg.height);
 
-    for (size_t y = 0; y < cloud.height; y++)
+    for (std::size_t y = 0; y < cloud.height; y++)
     {
-      for (size_t x = 0; x < cloud.width; x++, rgb_offset += point_step)
+      for (std::size_t x = 0; x < cloud.width; x++, rgb_offset += point_step)
       {
-        uint8_t * pixel = &(msg.data[y * msg.step + x * 3]);
-        memcpy (pixel, &(cloud.data[rgb_offset]), 3 * sizeof (uint8_t));
+        std::uint8_t * pixel = &(msg.data[y * msg.step + x * 3]);
+        std::copy(&cloud.data[rgb_offset], &cloud.data[rgb_offset] + 3, pixel);
       }
     }
   }
 }
-
-#endif  //#ifndef PCL_CONVERSIONS_H_
