@@ -1,9 +1,12 @@
 #include "pcl/recognition/face_detection/face_detector_data_provider.h"
 #include "pcl/recognition/face_detection/face_common.h"
+#include <pcl/common/point_tests.h> // for pcl::isFinite
 #include <pcl/common/time.h>
 #include <pcl/features/integral_image_normal.h>
 #include <pcl/io/pcd_io.h>
 
+#include <random>
+
 //Uncomment the following lines and set PCL_FACE_DETECTION_VIS_TRAINING_FDDP to 1
 //to visualize the training process and change the CMakeLists.txt accordingly.
 //#include <pcl/visualization/pcl_visualizer.h>
@@ -45,7 +48,7 @@ namespace pcl
 template<class FeatureType, class DataSet, class LabelType, class ExampleIndex, class NodeType>
 void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelType, ExampleIndex, NodeType>::initialize(std::string & data_dir)
 {
-  std::string start = "";
+  std::string start;
   std::string ext = std::string ("pcd");
   bf::path dir = data_dir;
 
@@ -76,26 +79,21 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
     }
   }
 
-  for (size_t i = 0; i < files.size (); i++)
+  for (const auto &filename : files)
   {
-    std::stringstream filestream;
-    filestream << data_dir << "/" << files[i];
-    std::string file = filestream.str ();
+    std::string file = data_dir + '/' + filename;
 
-    std::string pose_file (files[i]);
+    std::string pose_file (filename);
     boost::replace_all (pose_file, ".pcd", "_pose.txt");
 
     Eigen::Matrix4f pose_mat;
     pose_mat.setIdentity (4, 4);
 
-    std::stringstream filestream_pose;
-    filestream_pose << data_dir << "/" << pose_file;
-    pose_file = filestream_pose.str ();
+    pose_file = data_dir + '/' + pose_file;
 
-    bool result = readMatrixFromFile (pose_file, pose_mat);
-    if (result)
+    if (readMatrixFromFile (pose_file, pose_mat))
     {
-      Eigen::Vector3f ea = pose_mat.block<3, 3> (0, 0).eulerAngles (0, 1, 2);
+      Eigen::Vector3f ea = pose_mat.topLeftCorner<3, 3> ().eulerAngles (0, 1, 2);
       ea *= 57.2957795f; //transform it to degrees to do the binning
       int y = static_cast<int>(pcl_round ((ea[0] + static_cast<float>(std::abs (min_yaw))) / res_yaw));
       int p = static_cast<int>(pcl_round ((ea[1] + static_cast<float>(std::abs (min_pitch))) / res_pitch));
@@ -141,20 +139,21 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
   if (min_images_per_bin_ != -1)
   {
     std::cout << "Reducing unbalance of the dataset." << std::endl;
+    std::mt19937 rng((std::random_device()()));
     for (int i = 0; i < num_yaw; i++)
     {
       for (int j = 0; j < num_pitch; j++)
       {
         if (yaw_pitch_bins[i][j] >= min_images_per_bin_)
         {
-          std::random_shuffle (image_files_per_bin[i][j].begin (), image_files_per_bin[i][j].end ());
+          std::shuffle (image_files_per_bin[i][j].begin (), image_files_per_bin[i][j].end (), rng);
           image_files_per_bin[i][j].resize (min_images_per_bin_);
           yaw_pitch_bins[i][j] = min_images_per_bin_;
         }
 
-        for (size_t ii = 0; ii < image_files_per_bin[i][j].size (); ii++)
+        for (const auto &ii : image_files_per_bin[i][j])
         {
-          image_files_.push_back (image_files_per_bin[i][j][ii]);
+          image_files_.push_back (ii);
         }
       }
     }
@@ -172,8 +171,8 @@ template<class FeatureType, class DataSet, class LabelType, class ExampleIndex,
 void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelType, ExampleIndex, NodeType>::getDatasetAndLabels(DataSet & data_set,
     std::vector<LabelType> & label_data, std::vector<ExampleIndex> & examples)
 {
-  srand (static_cast<unsigned int>(time (NULL)));
-  std::random_shuffle (image_files_.begin (), image_files_.end ());
+  std::mt19937 rng((std::random_device()()));
+  std::shuffle (image_files_.begin (), image_files_.end (), rng);
   std::vector < std::string > files;
   files = image_files_;
   files.resize (std::min (num_images_, static_cast<int> (files.size ())));
@@ -188,7 +187,7 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
   pcl::visualization::PCLVisualizer vis("training");
 #endif
 
-  for (size_t j = 0; j < files.size (); j++)
+  for (std::size_t j = 0; j < files.size (); j++)
   {
 
 #if PCL_FACE_DETECTION_VIS_TRAINING_FDDP == 1
@@ -212,14 +211,14 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
 
     //crop images to remove as many NaNs as possible and reduce the memory footprint
     {
-      size_t min_col, min_row;
-      size_t max_col, max_row;
-      min_col = min_row = std::numeric_limits<size_t>::max ();
+      std::size_t min_col, min_row;
+      std::size_t max_col, max_row;
+      min_col = min_row = std::numeric_limits<std::size_t>::max ();
       max_col = max_row = 0;
 
-      for (size_t col = 0; col < loaded_cloud->width; col++)
+      for (std::size_t col = 0; col < loaded_cloud->width; col++)
       {
-        for (size_t row = 0; row < loaded_cloud->height; row++)
+        for (std::size_t row = 0; row < loaded_cloud->height; row++)
         {
           if (pcl::isFinite (loaded_cloud->at (col, row)))
           {
@@ -250,12 +249,12 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
     }
 
     //Compute integral image over depth
-    boost::shared_ptr < pcl::IntegralImage2D<float, 1> > integral_image_depth;
+    pcl::IntegralImage2D<float, 1>::Ptr integral_image_depth;
     integral_image_depth.reset (new pcl::IntegralImage2D<float, 1> (false));
 
     int element_stride = sizeof(pcl::PointXYZ) / sizeof(float);
     int row_stride = element_stride * cloud->width;
-    const float *data = reinterpret_cast<const float*> (&cloud->points[0]);
+    const float *data = reinterpret_cast<const float*> (cloud->data());
     integral_image_depth->setInput (data + 2, cloud->width, cloud->height, element_stride, row_stride);
 
     //Compute normals and normal integral images
@@ -263,8 +262,7 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
 
     if (USE_NORMALS_)
     {
-      typedef typename pcl::IntegralImageNormalEstimation<pcl::PointXYZ, pcl::Normal> NormalEstimator_;
-      NormalEstimator_ n3d;
+      pcl::IntegralImageNormalEstimation<pcl::PointXYZ, pcl::Normal> n3d;
       n3d.setNormalEstimationMethod (n3d.COVARIANCE_MATRIX);
       n3d.setInputCloud (cloud);
       n3d.setRadiusSearch (0.02);
@@ -277,37 +275,35 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
 
     int element_stride_normal = sizeof(pcl::Normal) / sizeof(float);
     int row_stride_normal = element_stride_normal * normals->width;
-    boost::shared_ptr < pcl::IntegralImage2D<float, 1> > integral_image_normal_x;
-    boost::shared_ptr < pcl::IntegralImage2D<float, 1> > integral_image_normal_y;
-    boost::shared_ptr < pcl::IntegralImage2D<float, 1> > integral_image_normal_z;
+    pcl::IntegralImage2D<float, 1>::Ptr integral_image_normal_x;
+    pcl::IntegralImage2D<float, 1>::Ptr integral_image_normal_y;
+    pcl::IntegralImage2D<float, 1>::Ptr integral_image_normal_z;
 
     if (USE_NORMALS_)
     {
       integral_image_normal_x.reset (new pcl::IntegralImage2D<float, 1> (false));
-      const float *data_nx = reinterpret_cast<const float*> (&normals->points[0]);
-      integral_image_normal_x->setInput (data_nx, normals->width, normals->height, element_stride_normal, row_stride_normal);
+      const float *data = reinterpret_cast<const float*> (normals->data());
+      integral_image_normal_x->setInput (data + 0, normals->width, normals->height, element_stride_normal, row_stride_normal);
 
       integral_image_normal_y.reset (new pcl::IntegralImage2D<float, 1> (false));
-      const float *data_ny = reinterpret_cast<const float*> (&normals->points[0]);
-      integral_image_normal_y->setInput (data_ny + 1, normals->width, normals->height, element_stride_normal, row_stride_normal);
+      integral_image_normal_y->setInput (data + 1, normals->width, normals->height, element_stride_normal, row_stride_normal);
 
       integral_image_normal_z.reset (new pcl::IntegralImage2D<float, 1> (false));
-      const float *data_nz = reinterpret_cast<const float*> (&normals->points[0]);
-      integral_image_normal_z->setInput (data_nz + 2, normals->width, normals->height, element_stride_normal, row_stride_normal);
+      integral_image_normal_z->setInput (data + 2, normals->width, normals->height, element_stride_normal, row_stride_normal);
     }
 
     //Using cloud labels estimate a 2D window from where to extract positive samples
     //Rest can be used to extract negative samples
-    size_t min_col, min_row;
-    size_t max_col, max_row;
-    min_col = min_row = std::numeric_limits<size_t>::max ();
+    std::size_t min_col, min_row;
+    std::size_t max_col, max_row;
+    min_col = min_row = std::numeric_limits<std::size_t>::max ();
     max_col = max_row = 0;
 
     //std::cout << cloud_labels->width << " " << cloud_labels->height << std::endl;
 
-    for (size_t col = 0; col < cloud_labels->width; col++)
+    for (std::size_t col = 0; col < cloud_labels->width; col++)
     {
-      for (size_t row = 0; row < cloud_labels->height; row++)
+      for (std::size_t row = 0; row < cloud_labels->height; row++)
       {
         if (cloud_labels->at (col, row).label == 1)
         {
@@ -330,20 +326,20 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
     pcl::PointCloud<pcl::PointXYZI>::Ptr cloud_intensity(new pcl::PointCloud<pcl::PointXYZI>);
     cloud_intensity->width = cloud->width;
     cloud_intensity->height = cloud->height;
-    cloud_intensity->points.resize(cloud->points.size());
+    cloud_intensity->points.resize(cloud->size());
     cloud_intensity->is_dense = cloud->is_dense;
 
-    for (int jjj = 0; jjj < static_cast<int>(cloud->points.size()); jjj++)
+    for (int jjj = 0; jjj < static_cast<int>(cloud->size()); jjj++)
     {
-      cloud_intensity->points[jjj].getVector4fMap() = cloud->points[jjj].getVector4fMap();
-      cloud_intensity->points[jjj].intensity = 0.f;
+      (*cloud_intensity)[jjj].getVector4fMap() = (*cloud)[jjj].getVector4fMap();
+      (*cloud_intensity)[jjj].intensity = 0.f;
       int row, col;
       col = jjj % cloud->width;
       row = jjj / cloud->width;
       //std::cout << row << " " << col << std::endl;
       if (check_inside(col, row, min_col, max_col, min_row, max_row))
       {
-        cloud_intensity->points[jjj].intensity = 1.f;
+        (*cloud_intensity)[jjj].intensity = 1.f;
       }
     }
 
@@ -358,7 +354,7 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
     pose_mat.setIdentity (4, 4);
     readMatrixFromFile (pose_file, pose_mat);
 
-    Eigen::Vector3f ea = pose_mat.block<3, 3> (0, 0).eulerAngles (0, 1, 2);
+    Eigen::Vector3f ea = pose_mat.topLeftCorner<3, 3> ().eulerAngles (0, 1, 2);
     Eigen::Vector3f trans_vector = Eigen::Vector3f (pose_mat (0, 3), pose_mat (1, 3), pose_mat (2, 3));
 
     pcl::PointXYZ center_point;
@@ -366,17 +362,16 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
     center_point.y = trans_vector[1];
     center_point.z = trans_vector[2];
 
-    int N_patches = patches_per_image_;
-    int pos_extracted = 0;
-    int neg_extracted = 0;
-    int w_size_2 = static_cast<int> (w_size_ / 2);
-
     //************************************************
     //2nd training style, fanelli's journal description
     //************************************************
     {
+      int N_patches = patches_per_image_;
+      int pos_extracted = 0;
+      int neg_extracted = 0;
+      int w_size_2 = static_cast<int> (w_size_ / 2);
 
-      typedef std::pair<int, int> pixelpair;
+      using pixelpair = std::pair<int, int>;
       std::vector < pixelpair > negative_p, positive_p;
       //get negative and positive indices to sample from
       for (int col = 0; col < (static_cast<int> (cloud_labels->width) - w_size_); col++)
@@ -400,17 +395,16 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
       }
 
       //shuffle and resize
-      std::random_shuffle (positive_p.begin (), positive_p.end ());
-      std::random_shuffle (negative_p.begin (), negative_p.end ());
+      std::shuffle (positive_p.begin (), positive_p.end (), rng);
+      std::shuffle (negative_p.begin (), negative_p.end (), rng);
       positive_p.resize (N_patches);
       negative_p.resize (N_patches);
 
       //extract positive patch
-      for (size_t p = 0; p < positive_p.size (); p++)
+      for (const auto &p : positive_p)
       {
-        int col, row;
-        col = positive_p[p].first;
-        row = positive_p[p].second;
+        int col = p.first;
+        int row = p.second;
 
         pcl::PointXYZ patch_center_point;
         patch_center_point.x = cloud->at (col + w_size_2, row + w_size_2).x;
@@ -431,7 +425,7 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
         te.wsize_ = w_size_;
 
         te.trans_ = center_point.getVector3fMap () - patch_center_point.getVector3fMap ();
-        te.trans_ *= 1000.f; //transform it to milimiters
+        te.trans_ *= 1000.f; //transform it to millimeters
         te.rot_ = ea;
         te.rot_ *= 57.2957795f; //transform it to degrees
 
@@ -466,11 +460,10 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
       sleep(2);
 #endif
 
-      for (size_t p = 0; p < negative_p.size (); p++)
+      for (const auto &p : negative_p)
       {
-        int col, row;
-        col = negative_p[p].first;
-        row = negative_p[p].second;
+        int col = p.first;
+        int row = p.second;
 
         TrainingExample te;
         te.iimages_.push_back (integral_image_depth);
@@ -527,7 +520,7 @@ void pcl::face_detection::FaceDetectorDataProvider<FeatureType, DataSet, LabelTy
 
   //train random forest and make persistent
   std::vector<int> example_indices;
-  for (size_t i = 0; i < labels.size (); i++)
+  for (std::size_t i = 0; i < labels.size (); i++)
     example_indices.push_back (static_cast<int> (i));
 
   label_data = labels;
