@@ -8,6 +8,7 @@
 #include "pcl/recognition/face_detection/rf_face_detector_trainer.h"
 #include "pcl/recognition/face_detection/face_common.h"
 #include "pcl/io/pcd_io.h"
+#include <pcl/memory.h>  // for dynamic_pointer_cast
 #include "pcl/ml/dt/decision_tree_trainer.h"
 #include "pcl/ml/dt/decision_tree_evaluator.h"
 #include "pcl/ml/dt/decision_forest_trainer.h"
@@ -22,6 +23,7 @@
 #include <pcl/recognition/hv/hv_papazov.h>
 #include <pcl/features/normal_3d.h>
 
+
 void pcl::RFFaceDetectorTrainer::trainWithDataProvider()
 {
 
@@ -32,7 +34,7 @@ void pcl::RFFaceDetectorTrainer::trainWithDataProvider()
   if (use_normals_)
     fhda.setNumChannels (4);
 
-  pcl::TernaryTreeMissingDataBranchEstimator * btt = new pcl::TernaryTreeMissingDataBranchEstimator ();
+  auto * btt = new pcl::TernaryTreeMissingDataBranchEstimator ();
   pcl::face_detection::PoseClassRegressionVarianceStatsEstimator<float, NodeType, std::vector<face_detection::TrainingExample>, int> rse (btt);
 
   std::vector<float> thresholds_;
@@ -49,8 +51,7 @@ void pcl::RFFaceDetectorTrainer::trainWithDataProvider()
   dft.setRandomFeaturesAtSplitNode (true);
   dft.setThresholds (thresholds_);
 
-  boost::shared_ptr < face_detection::FaceDetectorDataProvider<face_detection::FeatureType, std::vector<face_detection::TrainingExample>, float, int, NodeType>
-      > dtdp;
+  typename face_detection::FaceDetectorDataProvider<face_detection::FeatureType, std::vector<face_detection::TrainingExample>, float, int, NodeType>::Ptr dtdp;
   dtdp.reset (new face_detection::FaceDetectorDataProvider<face_detection::FeatureType, std::vector<face_detection::TrainingExample>, float, int, NodeType>);
   dtdp->setUseNormals (use_normals_);
   dtdp->setWSize (w_size_);
@@ -59,10 +60,7 @@ void pcl::RFFaceDetectorTrainer::trainWithDataProvider()
 
   dtdp->initialize (directory_);
 
-  boost::shared_ptr < pcl::DecisionTreeTrainerDataProvider<face_detection::FeatureType, std::vector<face_detection::TrainingExample>, float, int, NodeType>
-      > cast_dtdp;
-  cast_dtdp = boost::dynamic_pointer_cast
-      < pcl::DecisionTreeTrainerDataProvider<face_detection::FeatureType, std::vector<face_detection::TrainingExample>, float, int, NodeType> > (dtdp);
+  auto cast_dtdp = dynamic_pointer_cast<pcl::DecisionTreeTrainerDataProvider<face_detection::FeatureType, std::vector<face_detection::TrainingExample>, float, int, NodeType>> (dtdp);
   dft.setDecisionTreeDataProvider (cast_dtdp);
 
   pcl::DecisionForest<NodeType> forest;
@@ -85,12 +83,12 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
   std::vector < Eigen::Vector3f > clusters_mean;
   std::vector < std::vector<int> > votes_indices;
 
-  for (size_t i = 0; i < head_center_votes_.size (); i++)
+  for (std::size_t i = 0; i < head_center_votes_.size (); i++)
   {
     Eigen::Vector3f center_vote = head_center_votes_[i];
     std::vector<bool> valid_in_cluster (clusters_mean.size (), false);
     bool found = false;
-    for (size_t j = 0; j < clusters_mean.size () /*&& !found*/; j++)
+    for (std::size_t j = 0; j < clusters_mean.size () /*&& !found*/; j++)
     {
       float sq_norm = (clusters_mean[j] - center_vote).squaredNorm ();
       if (sq_norm < large_radius)
@@ -118,8 +116,8 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
 
     //get the largest biggest cluster and put if there
     int idx = -1;
-    size_t biggest_num = 0;
-    for (size_t j = 0; j < clusters_mean.size () /*&& !found*/; j++)
+    std::size_t biggest_num = 0;
+    for (std::size_t j = 0; j < clusters_mean.size () /*&& !found*/; j++)
     {
       if ((votes_indices[j].size () > biggest_num) && (valid_in_cluster[j]))
       {
@@ -141,7 +139,7 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
   std::cout << "Number of clusters:" << clusters_mean.size () << " votes:" << head_center_votes_.size () << std::endl;
 
   int valid = 0;
-  for (size_t i = 0; i < clusters_mean.size (); i++)
+  for (std::size_t i = 0; i < clusters_mean.size (); i++)
   {
     //ignore this cluster
     if (votes_indices[i].size () < min_votes_size_)
@@ -155,7 +153,7 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
       mean.setZero ();
       int good_votes = 0;
       new_cluster.clear ();
-      for (size_t j = 0; j < votes_indices[i].size (); j++)
+      for (std::size_t j = 0; j < votes_indices[i].size (); j++)
       {
         Eigen::Vector3f center_vote = head_center_votes_[votes_indices[i][j]];
         float sq_norm = (clusters_mean[i] - center_vote).squaredNorm ();
@@ -167,7 +165,7 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
         }
       }
 
-      mean = mean / static_cast<float> (good_votes);
+      mean /= static_cast<float> (good_votes);
       clusters_mean[i] = mean;
     }
 
@@ -185,18 +183,18 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
   head_clusters_rotation_.clear ();
   head_center_votes_clustered_.resize (clusters_mean.size ());
 
-  for (size_t i = 0; i < clusters_mean.size (); i++)
+  for (std::size_t i = 0; i < clusters_mean.size (); i++)
   {
     if (votes_indices[i].size () > min_votes_size_)
     {
       //compute rotation using the first less uncertain votes
       std::vector < std::pair<int, float> > uncertainty;
-      for (size_t j = 0; j < votes_indices[i].size (); j++)
+      for (const int &index : votes_indices[i])
       {
-        uncertainty.push_back (std::make_pair (votes_indices[i][j], uncertainties_[votes_indices[i][j]]));
+        uncertainty.emplace_back (index, uncertainties_[index]);
       }
 
-      std::sort (uncertainty.begin (), uncertainty.end (), boost::bind (&std::pair<int, float>::second, _1) < boost::bind (&std::pair<int, float>::second, _2));
+      std::sort (uncertainty.begin (), uncertainty.end (), [] (const auto& p1, const auto& p2) { return p1.second < p2.second; });
 
       Eigen::Vector3f rot;
       rot.setZero ();
@@ -206,19 +204,19 @@ void pcl::RFFaceDetectorTrainer::faceVotesClustering()
         rot += angle_votes_[uncertainty[j].first];
       }
 
-      rot = rot / static_cast<float> (num);
+      rot /= static_cast<float> (num);
 
       Eigen::Vector3f pos;
       pos.setZero ();
       for (int j = 0; j < num; j++)
         pos += head_center_votes_[uncertainty[j].first];
 
-      pos = pos / static_cast<float> (num);
+      pos /= static_cast<float> (num);
 
       head_clusters_centers_.push_back (pos); //clusters_mean[i]
       head_clusters_rotation_.push_back (rot);
 
-      for (size_t j = 0; j < votes_indices[i].size (); j++)
+      for (std::size_t j = 0; j < votes_indices[i].size (); j++)
       {
         head_center_votes_clustered_[i].push_back (head_center_votes_[votes_indices[i][j]]);
       }
@@ -273,12 +271,12 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
   pass_.filter (*cloud);
 
   //compute depth integral image
-  boost::shared_ptr<pcl::IntegralImage2D<float, 1> > integral_image_depth;
+  pcl::IntegralImage2D<float, 1>::Ptr integral_image_depth;
   integral_image_depth.reset (new pcl::IntegralImage2D<float, 1> (false));
 
   int element_stride = sizeof(pcl::PointXYZ) / sizeof(float);
   int row_stride = element_stride * cloud->width;
-  const float *data = reinterpret_cast<const float*> (&cloud->points[0]);
+  const float *data = reinterpret_cast<const float*> (cloud->data());
   integral_image_depth->setInput (data + 2, cloud->width, cloud->height, element_stride, row_stride);
 
   //Compute normals and normal integral images
@@ -286,7 +284,7 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
 
   if (use_normals_)
   {
-    typedef pcl::IntegralImageNormalEstimation<pcl::PointXYZ, pcl::Normal> NormalEstimator_;
+    using NormalEstimator_ = pcl::IntegralImageNormalEstimation<pcl::PointXYZ, pcl::Normal>;
     NormalEstimator_ n3d;
     n3d.setNormalEstimationMethod (n3d.COVARIANCE_MATRIX);
     n3d.setInputCloud (cloud);
@@ -297,23 +295,21 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
 
   int element_stride_normal = sizeof(pcl::Normal) / sizeof(float);
   int row_stride_normal = element_stride_normal * normals->width;
-  boost::shared_ptr<pcl::IntegralImage2D<float, 1> > integral_image_normal_x;
-  boost::shared_ptr<pcl::IntegralImage2D<float, 1> > integral_image_normal_y;
-  boost::shared_ptr<pcl::IntegralImage2D<float, 1> > integral_image_normal_z;
+  pcl::IntegralImage2D<float, 1>::Ptr integral_image_normal_x;
+  pcl::IntegralImage2D<float, 1>::Ptr integral_image_normal_y;
+  pcl::IntegralImage2D<float, 1>::Ptr integral_image_normal_z;
 
   if (use_normals_)
   {
     integral_image_normal_x.reset (new pcl::IntegralImage2D<float, 1> (false));
-    const float *data_nx = reinterpret_cast<const float*> (&normals->points[0]);
-    integral_image_normal_x->setInput (data_nx, normals->width, normals->height, element_stride_normal, row_stride_normal);
+    const float *datum = reinterpret_cast<const float*> (normals->data());
+    integral_image_normal_x->setInput (datum + 0, normals->width, normals->height, element_stride_normal, row_stride_normal);
 
     integral_image_normal_y.reset (new pcl::IntegralImage2D<float, 1> (false));
-    const float *data_ny = reinterpret_cast<const float*> (&normals->points[0]);
-    integral_image_normal_y->setInput (data_ny + 1, normals->width, normals->height, element_stride_normal, row_stride_normal);
+    integral_image_normal_y->setInput (datum + 1, normals->width, normals->height, element_stride_normal, row_stride_normal);
 
     integral_image_normal_z.reset (new pcl::IntegralImage2D<float, 1> (false));
-    const float *data_nz = reinterpret_cast<const float*> (&normals->points[0]);
-    integral_image_normal_z->setInput (data_nz + 2, normals->width, normals->height, element_stride_normal, row_stride_normal);
+    integral_image_normal_z->setInput (datum + 2, normals->width, normals->height, element_stride_normal, row_stride_normal);
   }
 
   {
@@ -326,13 +322,10 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
       fhda.setNumChannels (4);
 
     //pcl::BinaryTreeThresholdBasedBranchEstimator * btt = new pcl::BinaryTreeThresholdBasedBranchEstimator ();
-    pcl::TernaryTreeMissingDataBranchEstimator * btt = new pcl::TernaryTreeMissingDataBranchEstimator ();
+    auto * btt = new pcl::TernaryTreeMissingDataBranchEstimator ();
     face_detection::PoseClassRegressionVarianceStatsEstimator<float, NodeType, std::vector<face_detection::TrainingExample>, int> rse (btt);
 
-    std::vector<float> weights;
-    weights.resize (cloud->points.size ());
-    for (size_t i = 0; i < cloud->points.size (); i++)
-      weights[i] = 0;
+    std::vector<float> weights(cloud->size(), 0.f);
 
     int w_size_2 = static_cast<int> (w_size_ / 2);
 
@@ -369,16 +362,16 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
           std::vector<NodeType> leaves;
           dfe.evaluate (forest_, fhda, rse, eval_examples, 0, leaves);
 
-          for (size_t l = 0; l < leaves.size (); l++)
+          for (const auto &leaf : leaves)
           {
-            if (leaves[l].value >= thres_face_)
+            if (leaf.value >= thres_face_)
             {
-              if ((leaves[l].covariance_trans_.trace () + leaves[l].covariance_rot_.trace ()) > trans_max_variance_)
+              if ((leaf.covariance_trans_.trace () + leaf.covariance_rot_.trace ()) > trans_max_variance_)
                 continue;
 
-              Eigen::Vector3f head_center = Eigen::Vector3f (static_cast<float>(leaves[l].trans_mean_[0]),
-                                                             static_cast<float>(leaves[l].trans_mean_[1]),
-                                                             static_cast<float>(leaves[l].trans_mean_[2]));
+              Eigen::Vector3f head_center = Eigen::Vector3f (static_cast<float>(leaf.trans_mean_[0]),
+                                                             static_cast<float>(leaf.trans_mean_[1]),
+                                                             static_cast<float>(leaf.trans_mean_[2]));
               head_center *= 0.001f;
 
               pcl::PointXYZ patch_center_point;
@@ -393,7 +386,7 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
               if (!pcl::isFinite (ppp))
                 continue;
 
-              //this is a good leave
+              //this is a good leaf
               for (int j = te.col_; j < (te.col_ + w_size_); j++)
               {
                 for (int k = te.row_; k < (te.row_ + w_size_); k++)
@@ -402,11 +395,10 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
 
               head_center_votes_.push_back (head_center);
               float mult_fact = 0.0174532925f;
-              angle_votes_.push_back (
-                  Eigen::Vector3f (static_cast<float>(leaves[l].rot_mean_[0]) * mult_fact,
-                                   static_cast<float>(leaves[l].rot_mean_[1]) * mult_fact,
-                                   static_cast<float>(leaves[l].rot_mean_[2]) * mult_fact));
-              uncertainties_.push_back (static_cast<float>(leaves[l].covariance_trans_.trace () + leaves[l].covariance_rot_.trace ()));
+              angle_votes_.emplace_back(static_cast<float>(leaf.rot_mean_[0]) * mult_fact,
+                                   static_cast<float>(leaf.rot_mean_[1]) * mult_fact,
+                                   static_cast<float>(leaf.rot_mean_[2]) * mult_fact);
+              uncertainties_.push_back (static_cast<float>(leaf.covariance_trans_.trace () + leaf.covariance_rot_.trace ()));
             }
           }
         }
@@ -416,22 +408,22 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
     if (face_heat_map_)
     {
       face_heat_map_.reset (new pcl::PointCloud<pcl::PointXYZI>);
-      face_heat_map_->resize (cloud->points.size ());
+      face_heat_map_->resize (cloud->size ());
       face_heat_map_->height = 1;
-      face_heat_map_->width = static_cast<unsigned int>(cloud->points.size ());
+      face_heat_map_->width = cloud->size ();
       face_heat_map_->is_dense = false;
 
-      for (size_t i = 0; i < cloud->points.size (); i++)
+      for (std::size_t i = 0; i < cloud->size (); i++)
       {
-        face_heat_map_->points[i].getVector4fMap () = cloud->points[i].getVector4fMap ();
-        face_heat_map_->points[i].intensity = weights[i];
+        (*face_heat_map_)[i].getVector4fMap () = (*cloud)[i].getVector4fMap ();
+        (*face_heat_map_)[i].intensity = weights[i];
       }
     }
   }
 
   faceVotesClustering ();
 
-  if (pose_refinement_ && (head_clusters_centers_.size () > 0))
+  if (pose_refinement_ && (!head_clusters_centers_.empty ()))
   {
     Eigen::Matrix4f icp_trans;
     float max_distance = 0.015f;
@@ -441,7 +433,7 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
     pcl::PointCloud<pcl::Normal>::Ptr scene_normals (new pcl::PointCloud<pcl::Normal> ());
 
     {
-      typedef pcl::IntegralImageNormalEstimation<pcl::PointXYZ, pcl::Normal> NormalEstimator_;
+      using NormalEstimator_ = pcl::IntegralImageNormalEstimation<pcl::PointXYZ, pcl::Normal>;
       NormalEstimator_ n3d;
       n3d.setNormalEstimationMethod (n3d.COVARIANCE_MATRIX);
       n3d.setInputCloud (input_);
@@ -477,7 +469,7 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
     pcl::PointCloud<pcl::PointNormal>::Ptr output (new pcl::PointCloud<pcl::PointNormal> ());
 
     pcl::IterativeClosestPoint<pcl::PointNormal, pcl::PointNormal> reg;
-    for (size_t i = 0; i < head_clusters_centers_.size (); i++)
+    for (std::size_t i = 0; i < head_clusters_centers_.size (); i++)
     {
       Eigen::Matrix3f matrixxx;
 
@@ -487,7 +479,7 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
 
       Eigen::Matrix4f guess;
       guess.setIdentity ();
-      guess.block<3, 3> (0, 0) = matrixxx;
+      guess.topLeftCorner<3, 3> () = matrixxx;
       guess (0, 3) = head_clusters_centers_[i][0];
       guess (1, 3) = head_clusters_centers_[i][1];
       guess (2, 3) = head_clusters_centers_[i][2];
@@ -527,7 +519,7 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
       head_clusters_centers_[i][1] = icp_trans (1, 3);
       head_clusters_centers_[i][2] = icp_trans (2, 3);
 
-      Eigen::Vector3f ea = icp_trans.block<3, 3> (0, 0).eulerAngles (0, 1, 2);
+      Eigen::Vector3f ea = icp_trans.topLeftCorner<3, 3> ().eulerAngles (0, 1, 2);
       head_clusters_rotation_[i][0] = ea[0];
       head_clusters_rotation_[i][1] = ea[1];
       head_clusters_rotation_[i][2] = ea[2];
@@ -549,8 +541,8 @@ void pcl::RFFaceDetectorTrainer::detectFaces()
      papazov.verify ();
      papazov.getMask (mask_hv);
 
-     size_t valid=0;
-     for(size_t i=0; i < mask_hv.size(); i++) {
+     std::size_t valid=0;
+     for(std::size_t i=0; i < mask_hv.size(); i++) {
      if (!mask_hv[i])
      continue;
 
