@@ -38,18 +38,11 @@
  *
  */
 
-#include <gtest/gtest.h>
-#include <pcl/PCLPointCloud2.h>
-#include <pcl/point_traits.h>
+#include <pcl/test/gtest.h>
+#include <pcl/type_traits.h>
 #include <pcl/point_types.h>
-#include <pcl/common/io.h>
-#include <pcl/console/print.h>
 #include <pcl/io/ply_io.h>
-#include <pcl/io/ascii_io.h>
 #include <pcl/io/vtk_lib_io.h>
-#include <fstream>
-#include <locale>
-#include <stdexcept>
 
 std::string mesh_file_vtk_;
 
@@ -83,7 +76,7 @@ TEST (PCL, PLYPolygonMeshIO)
   EXPECT_EQ (verts_binary_vtk.size (), verts.size ());
   EXPECT_EQ (verts_ascii_pcl.size (), verts.size ());
   EXPECT_EQ (verts_binary_pcl.size (), verts.size ());
-  for (size_t i = 0; i < verts.size (); i++)
+  for (std::size_t i = 0; i < verts.size (); i++)
   {
     EXPECT_NEAR (verts_ascii_vtk.at (i).x, verts.at (i).x, 1E-2);
     EXPECT_NEAR (verts_ascii_vtk.at (i).y, verts.at (i).y, 1E-2);
@@ -102,13 +95,13 @@ TEST (PCL, PLYPolygonMeshIO)
   ASSERT_EQ (mesh_binary_vtk.polygons.size (), mesh.polygons.size ());
   ASSERT_EQ (mesh_ascii_pcl.polygons.size (), mesh.polygons.size ());
   ASSERT_EQ (mesh_binary_pcl.polygons.size (), mesh.polygons.size ());
-  for (size_t i = 0; i < mesh.polygons.size (); i++)
+  for (std::size_t i = 0; i < mesh.polygons.size (); i++)
   {
     ASSERT_EQ (mesh_ascii_vtk.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_binary_vtk.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_ascii_pcl.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_binary_pcl.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
-    for (size_t j = 0; j < mesh.polygons[i].vertices.size (); j++)
+    for (std::size_t j = 0; j < mesh.polygons[i].vertices.size (); j++)
     {
       EXPECT_EQ (mesh_ascii_vtk.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
       EXPECT_EQ (mesh_binary_vtk.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
@@ -116,6 +109,9 @@ TEST (PCL, PLYPolygonMeshIO)
       EXPECT_EQ (mesh_binary_pcl.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
     }
   }
+
+  remove ("test_mesh_ascii.ply");
+  remove ("test_mesh_binary.ply");
 }
 
 TEST (PCL, PLYPolygonMeshColoredIO)
@@ -131,14 +127,14 @@ TEST (PCL, PLYPolygonMeshColoredIO)
   pcl::fromPCLPointCloud2 (mesh.cloud, vertices_rgba);
   mesh_rgb.polygons = mesh.polygons;
   mesh_rgba.polygons = mesh.polygons;
-  for (size_t i = 0; i < vertices_rgb.size (); ++i)
+  for (std::size_t i = 0; i < vertices_rgb.size (); ++i)
   {
     pcl::PointXYZRGB &pt_rgb = vertices_rgb.at (i);
     pcl::PointXYZRGBA &pt_rgba = vertices_rgba.at (i);
-    pt_rgb.r = pt_rgba.r = static_cast<uint8_t> (rand () % 256);
-    pt_rgb.g = pt_rgba.g = static_cast<uint8_t> (rand () % 256);
-    pt_rgb.b = pt_rgba.b = static_cast<uint8_t> (rand () % 256);
-    pt_rgba.a = static_cast<uint8_t> (rand () % 256);
+    pt_rgb.r = pt_rgba.r = static_cast<std::uint8_t> (rand () % 256);
+    pt_rgb.g = pt_rgba.g = static_cast<std::uint8_t> (rand () % 256);
+    pt_rgb.b = pt_rgba.b = static_cast<std::uint8_t> (rand () % 256);
+    pt_rgba.a = static_cast<std::uint8_t> (rand () % 256);
   }
   pcl::toPCLPointCloud2 (vertices_rgb, mesh_rgb.cloud);
   pcl::toPCLPointCloud2 (vertices_rgba, mesh_rgba.cloud);
@@ -187,7 +183,7 @@ TEST (PCL, PLYPolygonMeshColoredIO)
   ASSERT_EQ (verts_rgba_ascii_pcl.size (), vertices_rgba.size ());
   ASSERT_EQ (verts_rgb_binary_pcl.size (), vertices_rgba.size ());
   ASSERT_EQ (verts_rgba_binary_pcl.size (), vertices_rgba.size ());
-  for (size_t i = 0; i < vertices_rgba.size (); i++)
+  for (std::size_t i = 0; i < vertices_rgba.size (); i++)
   {
     EXPECT_NEAR (verts_rgba_ascii_vtk.at (i).x, vertices_rgba.at (i).x, 1E-2);
     EXPECT_NEAR (verts_rgba_ascii_vtk.at (i).y, vertices_rgba.at (i).y, 1E-2);
@@ -243,13 +239,13 @@ TEST (PCL, PLYPolygonMeshColoredIO)
   ASSERT_EQ (mesh_rgba_ascii_vtk.polygons.size (), mesh.polygons.size ());
   ASSERT_EQ (mesh_rgb_binary_vtk.polygons.size (), mesh.polygons.size ());
   ASSERT_EQ (mesh_rgba_binary_vtk.polygons.size (), mesh.polygons.size ());
-  for (size_t i = 0; i < mesh.polygons.size (); i++)
+  for (std::size_t i = 0; i < mesh.polygons.size (); i++)
   {
     ASSERT_EQ (mesh_rgb_ascii_vtk.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_rgba_ascii_vtk.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_rgb_binary_vtk.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_rgba_binary_vtk.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
-    for (size_t j = 0; j < mesh.polygons[i].vertices.size (); j++)
+    for (std::size_t j = 0; j < mesh.polygons[i].vertices.size (); j++)
     {
       EXPECT_EQ (mesh_rgb_ascii_vtk.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
       EXPECT_EQ (mesh_rgba_ascii_vtk.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
@@ -262,13 +258,13 @@ TEST (PCL, PLYPolygonMeshColoredIO)
   ASSERT_EQ (mesh_rgba_ascii_pcl.polygons.size (), mesh.polygons.size ());
   ASSERT_EQ (mesh_rgb_binary_pcl.polygons.size (), mesh.polygons.size ());
   ASSERT_EQ (mesh_rgba_binary_pcl.polygons.size (), mesh.polygons.size ());
-  for (size_t i = 0; i < mesh.polygons.size (); i++)
+  for (std::size_t i = 0; i < mesh.polygons.size (); i++)
   {
     ASSERT_EQ (mesh_rgb_ascii_pcl.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_rgba_ascii_pcl.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_rgb_binary_pcl.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
     ASSERT_EQ (mesh_rgba_binary_pcl.polygons[i].vertices.size (), mesh.polygons[i].vertices.size ());
-    for (size_t j = 0; j < mesh.polygons[i].vertices.size (); j++)
+    for (std::size_t j = 0; j < mesh.polygons[i].vertices.size (); j++)
     {
       EXPECT_EQ (mesh_rgb_ascii_pcl.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
       EXPECT_EQ (mesh_rgba_ascii_pcl.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
@@ -276,8 +272,87 @@ TEST (PCL, PLYPolygonMeshColoredIO)
       EXPECT_EQ (mesh_rgba_binary_pcl.polygons[i].vertices[j], mesh.polygons[i].vertices[j]);
     }
   }
+
+  remove ("test_mesh_rgb_ascii.ply");
+  remove ("test_mesh_rgba_ascii.ply");
+  remove ("test_mesh_rgb_binary.ply");
+  remove ("test_mesh_rgba_binary.ply");
 }
 
+TEST (PCL, PLYPolygonMeshSpecificFieldOrder)
+{ // test a specific order of xyz, rgba, and normal fields
+  pcl::PolygonMesh mesh;
+  auto add_field = [](std::vector<pcl::PCLPointField>& fields, const std::string& name, const pcl::uindex_t offset, const std::uint8_t datatype)
+                   { fields.emplace_back(); fields.back().name = name; fields.back().offset = offset; fields.back().datatype = datatype; fields.back().count = 1; };
+  add_field(mesh.cloud.fields, "x", 0, pcl::PCLPointField::PointFieldTypes::FLOAT32);
+  add_field(mesh.cloud.fields, "y", 4, pcl::PCLPointField::PointFieldTypes::FLOAT32);
+  add_field(mesh.cloud.fields, "z", 8, pcl::PCLPointField::PointFieldTypes::FLOAT32);
+  add_field(mesh.cloud.fields, "normal_x", 12, pcl::PCLPointField::PointFieldTypes::FLOAT32);
+  add_field(mesh.cloud.fields, "normal_y", 16, pcl::PCLPointField::PointFieldTypes::FLOAT32);
+  add_field(mesh.cloud.fields, "normal_z", 20, pcl::PCLPointField::PointFieldTypes::FLOAT32);
+  add_field(mesh.cloud.fields, "rgba", 24, pcl::PCLPointField::PointFieldTypes::UINT32);
+  mesh.cloud.height = mesh.cloud.width = 1;
+  mesh.cloud.data.resize(28);
+  constexpr float x = 0.0, y = 1.0, z = 2.0, normal_x = 1.0, normal_y = 0.0, normal_z = 0.0;
+  constexpr std::uint32_t rgba = 0x326496;
+  memcpy(&mesh.cloud.data[0], &x, sizeof(float)); // NOLINT(readability-container-data-pointer)
+  memcpy(&mesh.cloud.data[4], &y, sizeof(float));
+  memcpy(&mesh.cloud.data[8], &z, sizeof(float));
+  memcpy(&mesh.cloud.data[12], &normal_x, sizeof(float));
+  memcpy(&mesh.cloud.data[16], &normal_y, sizeof(float));
+  memcpy(&mesh.cloud.data[20], &normal_z, sizeof(float));
+  memcpy(&mesh.cloud.data[24], &rgba, sizeof(std::uint32_t));
+
+  pcl::io::savePLYFileBinary("test_mesh_xyzrgbnormal_binary.ply", mesh);
+  pcl::PolygonMesh mesh_in_binary;
+  pcl::io::loadPLYFile("test_mesh_xyzrgbnormal_binary.ply", mesh_in_binary);
+  ASSERT_EQ (mesh.cloud.data.size(), mesh_in_binary.cloud.data.size());
+  for(std::size_t i=0; i<mesh.cloud.data.size(); ++i) {
+    EXPECT_EQ (mesh.cloud.data[i], mesh_in_binary.cloud.data[i]);
+  }
+  remove ("test_mesh_xyzrgbnormal_binary.ply");
+
+  pcl::io::savePLYFile("test_mesh_xyzrgbnormal_ascii.ply", mesh);
+  pcl::PolygonMesh mesh_in_ascii;
+  pcl::io::loadPLYFile("test_mesh_xyzrgbnormal_ascii.ply", mesh_in_ascii);
+  ASSERT_EQ (mesh.cloud.data.size(), mesh_in_ascii.cloud.data.size());
+  for(std::size_t i=0; i<mesh.cloud.data.size(); ++i) {
+    EXPECT_EQ (mesh.cloud.data[i], mesh_in_ascii.cloud.data[i]);
+  }
+  remove ("test_mesh_xyzrgbnormal_ascii.ply");
+}
+
+TEST (PCL, PLYPolygonMeshUintIndices)
+{
+  std::ofstream fs;
+  fs.open ("mesh_uint_indices.ply");
+  fs <<
+    "ply\n"
+    "format ascii 1.0\n"
+    "element vertex 3\n"
+    "property float x\n"
+    "property float y\n"
+    "property float z\n"
+    "element face 1\n"
+    "property list uchar uint vertex_indices\n"
+    "end_header\n"
+    "0.0 0.0 0.0\n"
+    "1.0 0.0 0.0\n"
+    "1.0 1.1 0.0\n"
+    "3 2 0 1\n";
+  fs.close();
+  pcl::PolygonMesh mesh;
+  int const res = pcl::io::loadPLYFile("mesh_uint_indices.ply", mesh);
+  EXPECT_NE (res, -1);
+  EXPECT_EQ (mesh.cloud.width, 3);
+  EXPECT_EQ (mesh.cloud.height, 1);
+  EXPECT_EQ (mesh.polygons.size(), 1);
+  EXPECT_EQ (mesh.polygons.front().vertices.size(), 3);
+
+  remove("mesh_uint_indices.ply");
+}
+
+
 /* ---[ */
 int
   main (int argc, char** argv)
