@@ -37,12 +37,11 @@
 *
 */
 
-#include <gtest/gtest.h>
+#include <pcl/test/gtest.h>
 
 #include <pcl/point_types.h>
 #include <pcl/io/pcd_io.h>
 #include <pcl/features/normal_3d.h>
-#include <pcl/features/fpfh.h>
 #include <pcl/registration/registration.h>
 #include <pcl/registration/icp.h>
 #include <pcl/registration/joint_icp.h>
@@ -55,19 +54,16 @@
 #include <pcl/registration/correspondence_rejection_sample_consensus.h>
 #include <pcl/registration/correspondence_rejection_surface_normal.h>
 #include <pcl/registration/correspondence_estimation_normal_shooting.h>
-#include <pcl/registration/ia_ransac.h>
 #include <pcl/registration/pyramid_feature_matching.h>
 #include <pcl/features/ppf.h>
 #include <pcl/registration/ppf_registration.h>
-#include <pcl/registration/ndt.h>
-#include <pcl/registration/sample_consensus_prerejective.h>
-// We need Histogram<2> to function, so we'll explicitely add kdtree_flann.hpp here
+#include <pcl/filters/voxel_grid.h>
+// We need Histogram<2> to function, so we'll explicitly add kdtree_flann.hpp here
 #include <pcl/kdtree/impl/kdtree_flann.hpp>
 //(pcl::Histogram<2>)
 
 using namespace pcl;
 using namespace pcl::io;
-using namespace std;
 
 PointCloud<PointXYZ> cloud_source, cloud_target, cloud_reg;
 PointCloud<PointXYZRGBA> cloud_with_color;
@@ -76,7 +72,7 @@ template <typename PointSource, typename PointTarget>
 class RegistrationWrapper : public Registration<PointSource, PointTarget>
 {
 public:
-  void computeTransformation (pcl::PointCloud<PointSource> &, const Eigen::Matrix4f&) { }
+  void computeTransformation (pcl::PointCloud<PointSource> &, const Eigen::Matrix4f&) override { }
 
   bool hasValidFeaturesTest ()
   {
@@ -91,8 +87,8 @@ public:
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, findFeatureCorrespondences)
 {
-  typedef Histogram<2> FeatureT;
-  typedef PointCloud<FeatureT> FeatureCloud;
+  using FeatureT = Histogram<2>;
+  using FeatureCloud = PointCloud<FeatureT>;
 
   RegistrationWrapper <PointXYZ, PointXYZ> reg;
 
@@ -107,25 +103,25 @@ TEST (PCL, findFeatureCorrespondences)
       FeatureT f;
       f.histogram[0] = x;
       f.histogram[1] = y;
-      feature0.points.push_back (f);
+      feature0.push_back (f);
 
       f.histogram[0] = x;
       f.histogram[1] = y - 2.5f;
-      feature1.points.push_back (f);
+      feature1.push_back (f);
 
       f.histogram[0] = x - 2.0f;
       f.histogram[1] = y + 1.5f;
-      feature2.points.push_back (f);
+      feature2.push_back (f);
 
       f.histogram[0] = x + 2.0f;
       f.histogram[1] = y + 1.5f;
-      feature3.points.push_back (f);
+      feature3.push_back (f);
     }
   }
-  feature0.width = static_cast<uint32_t> (feature0.points.size ());
-  feature1.width = static_cast<uint32_t> (feature1.points.size ());
-  feature2.width = static_cast<uint32_t> (feature2.points.size ());
-  feature3.width = static_cast<uint32_t> (feature3.points.size ());
+  feature0.width = feature0.size ();
+  feature1.width = feature1.size ();
+  feature2.width = feature2.size ();
+  feature3.width = feature3.size ();
 
   KdTreeFLANN<FeatureT> tree;
 
@@ -150,23 +146,60 @@ TEST (PCL, findFeatureCorrespondences)
 
   ASSERT_EQ ((int)indices.size (), 10);
   const int correct_values[] = {1197, 1248, 1249, 1299, 1300, 1301, 1302, 1350, 1351, 1401};
-  for (size_t i = 0; i < indices.size (); ++i)
+  for (std::size_t i = 0; i < indices.size (); ++i)
   {
     EXPECT_EQ (indices[i], correct_values[i]);
   }
 */
 }
 
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+// This test if the ICP algorithm can successfully find the transformation of a cloud that has been
+// moved 0.7 in x direction.
+// It indirectly test the KDTree doesn't get an empty input cloud, see #3624
+// It is more or less a copy of https://github.com/PointCloudLibrary/pcl/blob/cc7fe363c6463a0abc617b1e17e94ab4bd4169ef/doc/tutorials/content/sources/iterative_closest_point/iterative_closest_point.cpp
+TEST(PCL, ICP_translated)
+{
+  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_in(new pcl::PointCloud<pcl::PointXYZ>(5,1));
+  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_out(new pcl::PointCloud<pcl::PointXYZ>);
+
+  // Fill in the CloudIn data
+  for (auto& point : *cloud_in)
+  {
+    point.x = 1024 * rand() / (RAND_MAX + 1.0f);
+    point.y = 1024 * rand() / (RAND_MAX + 1.0f);
+    point.z = 1024 * rand() / (RAND_MAX + 1.0f);
+  }
+
+  *cloud_out = *cloud_in;
+
+  for (auto& point : *cloud_out)
+    point.x += 0.7f;
+
+  pcl::IterativeClosestPoint<pcl::PointXYZ, pcl::PointXYZ> icp;
+  icp.setInputSource(cloud_in);
+  icp.setInputTarget(cloud_out);
+
+  pcl::PointCloud<pcl::PointXYZ> Final;
+  icp.align(Final);
+
+  // Check that we have successfully converged
+  ASSERT_TRUE(icp.hasConverged());
+
+  // Test that the fitness score is below acceptable threshold
+  EXPECT_LT(icp.getFitnessScore(), 1e-6);
+
+  // Ensure that the translation found is within acceptable threshold.
+  EXPECT_NEAR(icp.getFinalTransformation()(0, 3), 0.7, 2e-3);
+  EXPECT_NEAR(icp.getFinalTransformation()(1, 3), 0.0, 2e-3);
+  EXPECT_NEAR(icp.getFinalTransformation()(2, 3), 0.0, 2e-3);
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, IterativeClosestPoint)
 {
   IterativeClosestPoint<PointXYZ, PointXYZ> reg;
   PointCloud<PointXYZ>::ConstPtr source (cloud_source.makeShared ());
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-  reg.setInputCloud (source);     // test for PCL_DEPRECATED
-  source = reg.getInputCloud ();  // test for PCL_DEPRECATED
-#pragma GCC diagnostic pop
   reg.setInputSource (source);
   reg.setInputTarget (cloud_target.makeShared ());
   reg.setMaximumIterations (50);
@@ -175,28 +208,39 @@ TEST (PCL, IterativeClosestPoint)
 
   // Register
   reg.align (cloud_reg);
-  EXPECT_EQ (int (cloud_reg.points.size ()), int (cloud_source.points.size ()));
+  EXPECT_EQ (cloud_reg.size (), cloud_source.size ());
 
-  //Eigen::Matrix4f transformation = reg.getFinalTransformation ();
-//  EXPECT_NEAR (transformation (0, 0), 0.8806,  1e-3);
-//  EXPECT_NEAR (transformation (0, 1), 0.036481287330389023, 1e-2);
-//  EXPECT_NEAR (transformation (0, 2), -0.4724, 1e-3);
-//  EXPECT_NEAR (transformation (0, 3), 0.03453, 1e-3);
-//
-//  EXPECT_NEAR (transformation (1, 0), -0.02354,  1e-3);
-//  EXPECT_NEAR (transformation (1, 1),  0.9992,   1e-3);
-//  EXPECT_NEAR (transformation (1, 2),  0.03326,  1e-3);
-//  EXPECT_NEAR (transformation (1, 3), -0.001519, 1e-3);
-//
-//  EXPECT_NEAR (transformation (2, 0),  0.4732,  1e-3);
-//  EXPECT_NEAR (transformation (2, 1), -0.01817, 1e-3);
-//  EXPECT_NEAR (transformation (2, 2),  0.8808,  1e-3);
-//  EXPECT_NEAR (transformation (2, 3),  0.04116, 1e-3);
-//
-//  EXPECT_EQ (transformation (3, 0), 0);
-//  EXPECT_EQ (transformation (3, 1), 0);
-//  EXPECT_EQ (transformation (3, 2), 0);
-//  EXPECT_EQ (transformation (3, 3), 1);
+  Eigen::Matrix4f transformation = reg.getFinalTransformation ();
+  EXPECT_NEAR (transformation (0, 0), 0.8806,  1e-3);
+  EXPECT_NEAR (transformation (0, 1), 0.036481287330389023, 1e-2);
+  EXPECT_NEAR (transformation (0, 2), -0.4724, 1e-3);
+  EXPECT_NEAR (transformation (0, 3), 0.03453, 1e-3);
+
+  EXPECT_NEAR (transformation (1, 0), -0.02354,  1e-3);
+  EXPECT_NEAR (transformation (1, 1),  0.9992,   1e-3);
+  EXPECT_NEAR (transformation (1, 2),  0.03326,  1e-3);
+  EXPECT_NEAR (transformation (1, 3), -0.001519, 1e-3);
+
+  EXPECT_NEAR (transformation (2, 0),  0.4732,  1e-3);
+  EXPECT_NEAR (transformation (2, 1), -0.01817, 1e-3);
+  EXPECT_NEAR (transformation (2, 2),  0.8808,  1e-3);
+  EXPECT_NEAR (transformation (2, 3),  0.04116, 1e-3);
+
+  EXPECT_EQ (transformation (3, 0), 0);
+  EXPECT_EQ (transformation (3, 1), 0);
+  EXPECT_EQ (transformation (3, 2), 0);
+  EXPECT_EQ (transformation (3, 3), 1);
+}
+
+TEST (PCL, IterativeClosestPointWithNormals)
+{
+  IterativeClosestPointWithNormals<PointNormal, PointNormal, float> reg_float;
+  reg_float.setUseSymmetricObjective(true);
+  EXPECT_TRUE(reg_float.getUseSymmetricObjective());
+
+  IterativeClosestPointWithNormals<PointNormal, PointNormal, double> reg_double;
+  reg_double.setUseSymmetricObjective(true);
+  EXPECT_TRUE(reg_double.getUseSymmetricObjective());
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -205,10 +249,10 @@ sampleRandomTransform (Eigen::Affine3f &trans, float max_angle, float max_trans)
 {
     srand(0);
     // Sample random transform
-    Eigen::Vector3f axis((float)rand() / RAND_MAX, (float)rand() / RAND_MAX, (float)rand() / RAND_MAX);
+    Eigen::Vector3f axis(static_cast<float>(rand()) / RAND_MAX, static_cast<float>(rand()) / RAND_MAX, static_cast<float>(rand()) / RAND_MAX);
     axis.normalize();
-    float angle = (float)rand() / RAND_MAX * max_angle;
-    Eigen::Vector3f translation((float)rand() / RAND_MAX, (float)rand() / RAND_MAX, (float)rand() / RAND_MAX);
+    float angle = static_cast<float>(rand()) / RAND_MAX * max_angle;
+    Eigen::Vector3f translation(static_cast<float>(rand()) / RAND_MAX, static_cast<float>(rand()) / RAND_MAX, static_cast<float>(rand()) / RAND_MAX);
     translation *= max_trans;
     Eigen::Affine3f rotation(Eigen::AngleAxis<float>(angle, axis));
     trans = Eigen::Translation3f(translation) * rotation;
@@ -229,8 +273,8 @@ TEST (PCL, IterativeClosestPointWithRejectors)
   pcl::registration::CorrespondenceRejectorSampleConsensus<PointXYZ>::Ptr rej_samp (new pcl::registration::CorrespondenceRejectorSampleConsensus<PointXYZ>);
   reg.addCorrespondenceRejector (rej_samp);
 
-  size_t ntransforms = 10;
-  for (size_t t = 0; t < ntransforms; t++)
+  std::size_t ntransforms = 10;
+  for (std::size_t t = 0; t < ntransforms; t++)
   {
     // Sample a fixed offset between cloud pairs
     Eigen::Affine3f delta_transform;
@@ -273,14 +317,14 @@ TEST (PCL, JointIterativeClosestPoint)
   reg.setMaxCorrespondenceDistance (0.25); // Making sure the correspondence distance > the max translation
   // Add a median distance rejector
   pcl::registration::CorrespondenceRejectorMedianDistance::Ptr rej_med (new pcl::registration::CorrespondenceRejectorMedianDistance);
-  rej_med->setMedianFactor (4.0);
+  rej_med->setMedianFactor (8.0);
   reg.addCorrespondenceRejector (rej_med);
   // Also add a SaC rejector
   pcl::registration::CorrespondenceRejectorSampleConsensus<PointXYZ>::Ptr rej_samp (new pcl::registration::CorrespondenceRejectorSampleConsensus<PointXYZ>);
   reg.addCorrespondenceRejector (rej_samp);
 
-  size_t ntransforms = 10;
-  for (size_t t = 0; t < ntransforms; t++)
+  std::size_t ntransforms = 10;
+  for (std::size_t t = 0; t < ntransforms; t++)
   {
 
     // Sample a fixed offset between cloud pairs
@@ -288,8 +332,8 @@ TEST (PCL, JointIterativeClosestPoint)
     // No rotation, since at a random offset this could make it converge to a wrong (but still reasonable) result
     sampleRandomTransform (delta_transform, 0., 0.10);
     // Make a few transformed versions of the data, plus noise
-    size_t nclouds = 5;
-    for (size_t i = 0; i < nclouds; i++)
+    std::size_t nclouds = 5;
+    for (std::size_t i = 0; i < nclouds; i++)
     {
       PointCloud<PointXYZ>::ConstPtr source (cloud_source.makeShared ());
       // Sample random global transform for each pair
@@ -323,7 +367,7 @@ TEST (PCL, JointIterativeClosestPoint)
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, IterativeClosestPointNonLinear)
 {
-  typedef PointXYZRGB PointT;
+  using PointT = PointXYZRGB;
   PointCloud<PointT>::Ptr temp_src (new PointCloud<PointT>);
   copyPointCloud (cloud_source, *temp_src);
   PointCloud<PointT>::Ptr temp_tgt (new PointCloud<PointT>);
@@ -338,7 +382,7 @@ TEST (PCL, IterativeClosestPointNonLinear)
 
   // Register
   reg.align (output);
-  EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+  EXPECT_EQ (output.size (), cloud_source.size ());
   // We get different results on 32 vs 64-bit systems.  To address this, we've removed the explicit output test
   // on the transformation matrix.  Instead, we're testing to make sure the algorithm converges to a sufficiently
   // low error by checking the fitness score.
@@ -385,7 +429,7 @@ TEST (PCL, IterativeClosestPointNonLinear)
 
     // Register
     reg.align (output);
-    EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+    EXPECT_EQ (output.size (), cloud_source.size ());
     EXPECT_LT (reg.getFitnessScore (), 0.001);
   }
 
@@ -394,7 +438,7 @@ TEST (PCL, IterativeClosestPointNonLinear)
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, IterativeClosestPoint_PointToPlane)
 {
-  typedef PointNormal PointT;
+  using PointT = PointNormal;
   PointCloud<PointT>::Ptr src (new PointCloud<PointT>);
   copyPointCloud (cloud_source, *src);
   PointCloud<PointT>::Ptr tgt (new PointCloud<PointT>);
@@ -408,8 +452,8 @@ TEST (PCL, IterativeClosestPoint_PointToPlane)
   norm_est.compute (*tgt);
 
   IterativeClosestPoint<PointT, PointT> reg;
-  typedef registration::TransformationEstimationPointToPlane<PointT, PointT> PointToPlane;
-  boost::shared_ptr<PointToPlane> point_to_plane (new PointToPlane);
+  using PointToPlane = registration::TransformationEstimationPointToPlane<PointT, PointT>;
+  PointToPlane::Ptr point_to_plane (new PointToPlane);
   reg.setTransformationEstimation (point_to_plane);
   reg.setInputSource (src);
   reg.setInputTarget (tgt);
@@ -425,14 +469,14 @@ TEST (PCL, IterativeClosestPoint_PointToPlane)
 
   // Register
   reg.align (output);
-  EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+  EXPECT_EQ (output.size (), cloud_source.size ());
   EXPECT_LT (reg.getFitnessScore (), 0.005);
 
   // Check again, for all possible caching schemes
   for (int iter = 0; iter < 4; iter++)
   {
-    bool force_cache = (bool) iter/2;
-    bool force_cache_reciprocal = (bool) iter%2;
+    bool force_cache = static_cast<bool> (iter/2);
+    bool force_cache_reciprocal = static_cast<bool> (iter%2);
     pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
     // Ensure that, when force_cache is not set, we are robust to the wrong input
     if (force_cache)
@@ -446,7 +490,7 @@ TEST (PCL, IterativeClosestPoint_PointToPlane)
 
     // Register
     reg.align (output);
-    EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+    EXPECT_EQ (output.size (), cloud_source.size ());
     EXPECT_LT (reg.getFitnessScore (), 0.005);
   }
 
@@ -483,7 +527,7 @@ TEST (PCL, IterativeClosestPoint_PointToPlane)
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, GeneralizedIterativeClosestPoint)
 {
-  typedef PointXYZ PointT;
+  using PointT = PointXYZ;
   PointCloud<PointT>::Ptr src (new PointCloud<PointT>);
   copyPointCloud (cloud_source, *src);
   PointCloud<PointT>::Ptr tgt (new PointCloud<PointT>);
@@ -498,14 +542,14 @@ TEST (PCL, GeneralizedIterativeClosestPoint)
 
   // Register
   reg.align (output);
-  EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+  EXPECT_EQ (output.size (), cloud_source.size ());
   EXPECT_LT (reg.getFitnessScore (), 0.0001);
 
   // Check again, for all possible caching schemes
   for (int iter = 0; iter < 4; iter++)
   {
-    bool force_cache = (bool) iter/2;
-    bool force_cache_reciprocal = (bool) iter%2;
+    bool force_cache = static_cast<bool> (iter/2);
+    bool force_cache_reciprocal = static_cast<bool> (iter%2);
     pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
     // Ensure that, when force_cache is not set, we are robust to the wrong input
     if (force_cache)
@@ -519,7 +563,7 @@ TEST (PCL, GeneralizedIterativeClosestPoint)
 
     // Register
     reg.align (output);
-    EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+    EXPECT_EQ (output.size (), cloud_source.size ());
     EXPECT_LT (reg.getFitnessScore (), 0.001);
   }
 
@@ -537,19 +581,81 @@ TEST (PCL, GeneralizedIterativeClosestPoint)
   reg_guess.setMaximumIterations (50);
   reg_guess.setTransformationEpsilon (1e-8);
   reg_guess.align (output, transform.matrix ());
-  EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+  EXPECT_EQ (output.size (), cloud_source.size ());
+  EXPECT_LT (reg.getFitnessScore (), 0.0001);
+}
+
+TEST (PCL, GeneralizedIterativeClosestPointBFGS)
+{ // same as above, but uses BFGS optimizer
+  using PointT = PointXYZ;
+  PointCloud<PointT>::Ptr src (new PointCloud<PointT>);
+  copyPointCloud (cloud_source, *src);
+  PointCloud<PointT>::Ptr tgt (new PointCloud<PointT>);
+  copyPointCloud (cloud_target, *tgt);
+  PointCloud<PointT> output;
+
+  GeneralizedIterativeClosestPoint<PointT, PointT> reg;
+  reg.useBFGS ();
+  reg.setInputSource (src);
+  reg.setInputTarget (tgt);
+  reg.setMaximumIterations (50);
+  reg.setTransformationEpsilon (1e-8);
+
+  // Register
+  reg.align (output);
+  EXPECT_EQ (output.size (), cloud_source.size ());
+  EXPECT_LT (reg.getFitnessScore (), 0.0001);
+
+  // Check again, for all possible caching schemes
+  for (int iter = 0; iter < 4; iter++)
+  {
+    bool force_cache = static_cast<bool> (iter/2);
+    bool force_cache_reciprocal = static_cast<bool> (iter%2);
+    pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
+    // Ensure that, when force_cache is not set, we are robust to the wrong input
+    if (force_cache)
+      tree->setInputCloud (tgt);
+    reg.setSearchMethodTarget (tree, force_cache);
+
+    pcl::search::KdTree<PointT>::Ptr tree_recip (new pcl::search::KdTree<PointT>);
+    if (force_cache_reciprocal)
+      tree_recip->setInputCloud (src);
+    reg.setSearchMethodSource (tree_recip, force_cache_reciprocal);
+
+    // Register
+    reg.align (output);
+    EXPECT_EQ (output.size (), cloud_source.size ());
+    EXPECT_LT (reg.getFitnessScore (), 0.001);
+  }
+
+  // Test guess matrix
+  Eigen::Isometry3f transform = Eigen::Isometry3f (Eigen::AngleAxisf (0.25 * M_PI, Eigen::Vector3f::UnitX ())
+                                                 * Eigen::AngleAxisf (0.50 * M_PI, Eigen::Vector3f::UnitY ())
+                                                 * Eigen::AngleAxisf (0.33 * M_PI, Eigen::Vector3f::UnitZ ()));
+  transform.translation () = Eigen::Vector3f (0.1, 0.2, 0.3);
+  PointCloud<PointT>::Ptr transformed_tgt (new PointCloud<PointT>);
+  pcl::transformPointCloud (*tgt, *transformed_tgt, transform.matrix ()); // transformed_tgt is now a copy of tgt with a transformation matrix applied
+
+  GeneralizedIterativeClosestPoint<PointT, PointT> reg_guess;
+  reg_guess.useBFGS ();
+  reg_guess.setInputSource (src);
+  reg_guess.setInputTarget (transformed_tgt);
+  reg_guess.setMaximumIterations (50);
+  reg_guess.setTransformationEpsilon (1e-8);
+  reg_guess.align (output, transform.matrix ());
+  EXPECT_EQ (output.size (), cloud_source.size ());
   EXPECT_LT (reg.getFitnessScore (), 0.0001);
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, GeneralizedIterativeClosestPoint6D)
 {
-  typedef PointXYZRGBA PointT;
+  using PointT = PointXYZRGBA;
   Eigen::Affine3f delta_transform;
   PointCloud<PointT>::Ptr src_full (new PointCloud<PointT>);
   copyPointCloud (cloud_with_color, *src_full);
   PointCloud<PointT>::Ptr tgt_full (new PointCloud<PointT>);
-  sampleRandomTransform (delta_transform, M_PI/0.1, .03);
+  sampleRandomTransform (delta_transform, 0.25*M_PI, .03);
   pcl::transformPointCloud (cloud_with_color, *tgt_full, delta_transform);
   PointCloud<PointT> output;
 
@@ -571,14 +677,14 @@ TEST (PCL, GeneralizedIterativeClosestPoint6D)
 
   // Register
   reg.align (output);
-  EXPECT_EQ (int (output.points.size ()), int (src->points.size ()));
-  EXPECT_LT (reg.getFitnessScore (), 0.003);
+  EXPECT_EQ (output.size (), src->size ());
+  EXPECT_LT (reg.getFitnessScore (), 1e-4);
 
   // Check again, for all possible caching schemes
   for (int iter = 0; iter < 4; iter++)
   {
-    bool force_cache = (bool) iter/2;
-    bool force_cache_reciprocal = (bool) iter%2;
+    bool force_cache = static_cast<bool> (iter/2);
+    bool force_cache_reciprocal = static_cast<bool> (iter%2);
     pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
     // Ensure that, when force_cache is not set, we are robust to the wrong input
     if (force_cache)
@@ -592,236 +698,8 @@ TEST (PCL, GeneralizedIterativeClosestPoint6D)
 
     // Register
     reg.align (output);
-    EXPECT_EQ (int (output.points.size ()), int (src->points.size ()));
-    EXPECT_LT (reg.getFitnessScore (), 0.003);
-  }
-}
-
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-TEST (PCL, NormalDistributionsTransform)
-{
-  typedef PointNormal PointT;
-  PointCloud<PointT>::Ptr src (new PointCloud<PointT>);
-  copyPointCloud (cloud_source, *src);
-  PointCloud<PointT>::Ptr tgt (new PointCloud<PointT>);
-  copyPointCloud (cloud_target, *tgt);
-  PointCloud<PointT> output;
-
-  NormalDistributionsTransform<PointT, PointT> reg;
-  reg.setStepSize (0.05);
-  reg.setResolution (0.025f);
-  reg.setInputSource (src);
-  reg.setInputTarget (tgt);
-  reg.setMaximumIterations (50);
-  reg.setTransformationEpsilon (1e-8);
-  // Register
-  reg.align (output);
-  EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
-  EXPECT_LT (reg.getFitnessScore (), 0.001);
-
-  // Check again, for all possible caching schemes
-  for (int iter = 0; iter < 4; iter++)
-  {
-    bool force_cache = (bool) iter/2;
-    bool force_cache_reciprocal = (bool) iter%2;
-    pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
-    // Ensure that, when force_cache is not set, we are robust to the wrong input
-    if (force_cache)
-      tree->setInputCloud (tgt);
-    reg.setSearchMethodTarget (tree, force_cache);
-
-    pcl::search::KdTree<PointT>::Ptr tree_recip (new pcl::search::KdTree<PointT>);
-    if (force_cache_reciprocal)
-      tree_recip->setInputCloud (src);
-    reg.setSearchMethodSource (tree_recip, force_cache_reciprocal);
-
-    // Register
-    reg.align (output);
-    EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
-    EXPECT_LT (reg.getFitnessScore (), 0.001);
-  }
-}
-
-
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-TEST (PCL, SampleConsensusInitialAlignment)
-{
-  // Transform the source cloud by a large amount
-  Eigen::Vector3f initial_offset (100, 0, 0);
-  float angle = static_cast<float> (M_PI) / 2.0f;
-  Eigen::Quaternionf initial_rotation (cos (angle / 2), 0, 0, sin (angle / 2));
-  PointCloud<PointXYZ> cloud_source_transformed;
-  transformPointCloud (cloud_source, cloud_source_transformed, initial_offset, initial_rotation);
-
-  // Create shared pointers
-  PointCloud<PointXYZ>::Ptr cloud_source_ptr, cloud_target_ptr;
-  cloud_source_ptr = cloud_source_transformed.makeShared ();
-  cloud_target_ptr = cloud_target.makeShared ();
-
-  // Initialize estimators for surface normals and FPFH features
-  search::KdTree<PointXYZ>::Ptr tree (new search::KdTree<PointXYZ>);
-
-  NormalEstimation<PointXYZ, Normal> norm_est;
-  norm_est.setSearchMethod (tree);
-  norm_est.setRadiusSearch (0.05);
-  PointCloud<Normal> normals;
-
-  FPFHEstimation<PointXYZ, Normal, FPFHSignature33> fpfh_est;
-  fpfh_est.setSearchMethod (tree);
-  fpfh_est.setRadiusSearch (0.05);
-  PointCloud<FPFHSignature33> features_source, features_target;
-
-  // Estimate the FPFH features for the source cloud
-  norm_est.setInputCloud (cloud_source_ptr);
-  norm_est.compute (normals);
-  fpfh_est.setInputCloud (cloud_source_ptr);
-  fpfh_est.setInputNormals (normals.makeShared ());
-  fpfh_est.compute (features_source);
-
-  // Estimate the FPFH features for the target cloud
-  norm_est.setInputCloud (cloud_target_ptr);
-  norm_est.compute (normals);
-  fpfh_est.setInputCloud (cloud_target_ptr);
-  fpfh_est.setInputNormals (normals.makeShared ());
-  fpfh_est.compute (features_target);
-
-  // Initialize Sample Consensus Initial Alignment (SAC-IA)
-  SampleConsensusInitialAlignment<PointXYZ, PointXYZ, FPFHSignature33> reg;
-  reg.setMinSampleDistance (0.05f);
-  reg.setMaxCorrespondenceDistance (0.1);
-  reg.setMaximumIterations (1000);
-
-  reg.setInputSource (cloud_source_ptr);
-  reg.setInputTarget (cloud_target_ptr);
-  reg.setSourceFeatures (features_source.makeShared ());
-  reg.setTargetFeatures (features_target.makeShared ());
-
-  // Register
-  reg.align (cloud_reg);
-  EXPECT_EQ (int (cloud_reg.points.size ()), int (cloud_source.points.size ()));
-  EXPECT_LT (reg.getFitnessScore (), 0.0005);
-
-  // Check again, for all possible caching schemes
-  typedef pcl::PointXYZ PointT;
-  for (int iter = 0; iter < 4; iter++)
-  {
-    bool force_cache = (bool) iter/2;
-    bool force_cache_reciprocal = (bool) iter%2;
-    pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
-    // Ensure that, when force_cache is not set, we are robust to the wrong input
-    if (force_cache)
-      tree->setInputCloud (cloud_target_ptr);
-    reg.setSearchMethodTarget (tree, force_cache);
-
-    pcl::search::KdTree<PointT>::Ptr tree_recip (new pcl::search::KdTree<PointT>);
-    if (force_cache_reciprocal)
-      tree_recip->setInputCloud (cloud_source_ptr);
-    reg.setSearchMethodSource(tree_recip, force_cache_reciprocal);
-
-    // Register
-    reg.align (cloud_reg);
-    EXPECT_EQ (int (cloud_reg.points.size ()), int (cloud_source.points.size ()));
-    EXPECT_LT (reg.getFitnessScore (), 0.0005);
-  }
-}
-
-
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-TEST (PCL, SampleConsensusPrerejective)
-{
-  /*
-   * This test is a near-exact copy of the SampleConsensusInitialAlignment test,
-   * with the only modifications that:
-   *   1) the number of iterations is increased 1000 --> 5000
-   *   2) the feature correspondence randomness (the number of kNNs) is decreased 10 --> 2
-   */
-
-  // Transform the source cloud by a large amount
-  Eigen::Vector3f initial_offset (100, 0, 0);
-  float angle = static_cast<float> (M_PI) / 2.0f;
-  Eigen::Quaternionf initial_rotation (cos (angle / 2), 0, 0, sin (angle / 2));
-  PointCloud<PointXYZ> cloud_source_transformed;
-  transformPointCloud (cloud_source, cloud_source_transformed, initial_offset, initial_rotation);
-
-  // Create shared pointers
-  PointCloud<PointXYZ>::Ptr cloud_source_ptr, cloud_target_ptr;
-  cloud_source_ptr = cloud_source_transformed.makeShared ();
-  cloud_target_ptr = cloud_target.makeShared ();
-
-  // Initialize estimators for surface normals and FPFH features
-  search::KdTree<PointXYZ>::Ptr tree (new search::KdTree<PointXYZ>);
-
-  // Normal estimator
-  NormalEstimation<PointXYZ, Normal> norm_est;
-  norm_est.setSearchMethod (tree);
-  norm_est.setRadiusSearch (0.005);
-  PointCloud<Normal> normals;
-
-  // FPFH estimator
-  FPFHEstimation<PointXYZ, Normal, FPFHSignature33> fpfh_est;
-  fpfh_est.setSearchMethod (tree);
-  fpfh_est.setRadiusSearch (0.05);
-  PointCloud<FPFHSignature33> features_source, features_target;
-
-  // Estimate the normals and the FPFH features for the source cloud
-  norm_est.setInputCloud (cloud_source_ptr);
-  norm_est.compute (normals);
-  fpfh_est.setInputCloud (cloud_source_ptr);
-  fpfh_est.setInputNormals (normals.makeShared ());
-  fpfh_est.compute (features_source);
-
-  // Estimate the normals and the FPFH features for the target cloud
-  norm_est.setInputCloud (cloud_target_ptr);
-  norm_est.compute (normals);
-  fpfh_est.setInputCloud (cloud_target_ptr);
-  fpfh_est.setInputNormals (normals.makeShared ());
-  fpfh_est.compute (features_target);
-
-  // Initialize Sample Consensus Prerejective with 5x the number of iterations and 1/5 feature kNNs as SAC-IA
-  SampleConsensusPrerejective<PointXYZ, PointXYZ, FPFHSignature33> reg;
-  reg.setMaxCorrespondenceDistance (0.1);
-  reg.setMaximumIterations (5000);
-  reg.setSimilarityThreshold (0.6f);
-  reg.setCorrespondenceRandomness (2);
-
-  // Set source and target cloud/features
-  reg.setInputSource (cloud_source_ptr);
-  reg.setInputTarget (cloud_target_ptr);
-  reg.setSourceFeatures (features_source.makeShared ());
-  reg.setTargetFeatures (features_target.makeShared ());
-
-  // Register
-  reg.align (cloud_reg);
-
-  // Check output consistency and quality of alignment
-  EXPECT_EQ (static_cast<int> (cloud_reg.points.size ()), static_cast<int> (cloud_source.points.size ()));
-  float inlier_fraction = static_cast<float> (reg.getInliers ().size ()) / static_cast<float> (cloud_source.points.size ());
-  EXPECT_GT (inlier_fraction, 0.95f);
-
-  // Check again, for all possible caching schemes
-  typedef pcl::PointXYZ PointT;
-  for (int iter = 0; iter < 4; iter++)
-  {
-    bool force_cache = (bool) iter/2;
-    bool force_cache_reciprocal = (bool) iter%2;
-    pcl::search::KdTree<PointT>::Ptr tree(new pcl::search::KdTree<PointT>);
-    // Ensure that, when force_cache is not set, we are robust to the wrong input
-    if (force_cache)
-      tree->setInputCloud (cloud_target_ptr);
-    reg.setSearchMethodTarget (tree, force_cache);
-
-    pcl::search::KdTree<PointT>::Ptr tree_recip (new pcl::search::KdTree<PointT>);
-    if (force_cache_reciprocal)
-      tree_recip->setInputCloud (cloud_source_ptr);
-    reg.setSearchMethodSource(tree_recip, force_cache_reciprocal);
-
-    // Register
-    reg.align (cloud_reg);
-
-    // Check output consistency and quality of alignment
-    EXPECT_EQ (int (cloud_reg.points.size ()), int (cloud_source.points.size ()));
-    inlier_fraction = static_cast<float> (reg.getInliers ().size ()) / static_cast<float> (cloud_source.points.size ());
-    EXPECT_GT (inlier_fraction, 0.95f);
+    EXPECT_EQ (output.size (), src->size ());
+    EXPECT_LT (reg.getFitnessScore (), 1e-4);
   }
 }
 
@@ -858,11 +736,11 @@ TEST (PCL, PyramidFeatureHistogram)
   ppf_estimator.compute (*ppf_signature_target);
 
 
-  vector<pair<float, float> > dim_range_input, dim_range_target;
-  for (size_t i = 0; i < 3; ++i) dim_range_input.push_back (pair<float, float> (static_cast<float> (-M_PI), static_cast<float> (M_PI)));
-  dim_range_input.push_back (pair<float, float> (0.0f, 1.0f));
-  for (size_t i = 0; i < 3; ++i) dim_range_target.push_back (pair<float, float> (static_cast<float> (-M_PI) * 10.0f, static_cast<float> (M_PI) * 10.0f));
-  dim_range_target.push_back (pair<float, float> (0.0f, 50.0f));
+  std::vector<std::pair<float, float> > dim_range_input, dim_range_target;
+  for (std::size_t i = 0; i < 3; ++i) dim_range_input.emplace_back(static_cast<float> (-M_PI), static_cast<float> (M_PI));
+  dim_range_input.emplace_back(0.0f, 1.0f);
+  for (std::size_t i = 0; i < 3; ++i) dim_range_target.emplace_back(static_cast<float> (-M_PI) * 10.0f, static_cast<float> (M_PI) * 10.0f);
+  dim_range_target.emplace_back(0.0f, 50.0f);
 
 
   PyramidFeatureHistogram<PPFSignature>::Ptr pyramid_source (new PyramidFeatureHistogram<PPFSignature> ()),
@@ -878,11 +756,11 @@ TEST (PCL, PyramidFeatureHistogram)
   pyramid_target->compute ();
 
   float similarity_value = PyramidFeatureHistogram<PPFSignature>::comparePyramidFeatureHistograms (pyramid_source, pyramid_target);
-  EXPECT_NEAR (similarity_value, 0.74101555347442627, 1e-4);
+  EXPECT_NEAR (similarity_value, 0.738492727, 1e-4);
 
-  vector<pair<float, float> > dim_range_target2;
-  for (size_t i = 0; i < 3; ++i) dim_range_target2.push_back (pair<float, float> (static_cast<float> (-M_PI) * 5.0f, static_cast<float> (M_PI) * 5.0f));
-    dim_range_target2.push_back (pair<float, float> (0.0f, 20.0f));
+  std::vector<std::pair<float, float> > dim_range_target2;
+  for (std::size_t i = 0; i < 3; ++i) dim_range_target2.emplace_back(static_cast<float> (-M_PI) * 5.0f, static_cast<float> (M_PI) * 5.0f);
+    dim_range_target2.emplace_back(0.0f, 20.0f);
 
   pyramid_source->setTargetDimensionRange (dim_range_target2);
   pyramid_source->compute ();
@@ -891,12 +769,12 @@ TEST (PCL, PyramidFeatureHistogram)
   pyramid_target->compute ();
 
   float similarity_value2 = PyramidFeatureHistogram<PPFSignature>::comparePyramidFeatureHistograms (pyramid_source, pyramid_target);
-  EXPECT_NEAR (similarity_value2, 0.80097091197967529, 1e-4);
+  EXPECT_NEAR (similarity_value2, 0.798465133, 1e-4);
 
 
-  vector<pair<float, float> > dim_range_target3;
-  for (size_t i = 0; i < 3; ++i) dim_range_target3.push_back (pair<float, float> (static_cast<float> (-M_PI) * 2.0f, static_cast<float> (M_PI) * 2.0f));
-  dim_range_target3.push_back (pair<float, float> (0.0f, 10.0f));
+  std::vector<std::pair<float, float> > dim_range_target3;
+  for (std::size_t i = 0; i < 3; ++i) dim_range_target3.emplace_back(static_cast<float> (-M_PI) * 2.0f, static_cast<float> (M_PI) * 2.0f);
+  dim_range_target3.emplace_back(0.0f, 10.0f);
 
   pyramid_source->setTargetDimensionRange (dim_range_target3);
   pyramid_source->compute ();
@@ -905,21 +783,24 @@ TEST (PCL, PyramidFeatureHistogram)
   pyramid_target->compute ();
 
   float similarity_value3 = PyramidFeatureHistogram<PPFSignature>::comparePyramidFeatureHistograms (pyramid_source, pyramid_target);
-  EXPECT_NEAR (similarity_value3, 0.87623238563537598, 1e-3);
+  EXPECT_NEAR (similarity_value3, 0.873699546, 1e-3);
 }
 
-// Suat G: disabled, since the transformation does not look correct.
-// ToDo: update transformation from the ground truth.
-#if 0
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, PPFRegistration)
 {
-  // Transform the source cloud by a large amount
-  Eigen::Vector3f initial_offset (100, 0, 0);
-  float angle = M_PI/6;
-  Eigen::Quaternionf initial_rotation (cos (angle / 2), 0, 0, sin (angle / 2));
+  Eigen::Matrix4f bun0_to_bun4_groundtruth;
+  bun0_to_bun4_groundtruth <<
+    0.825336f, 0.000000f, -0.564642f, 0.037267f,
+    0.000000f, 1.000000f,  0.000000f, 0.000000f,
+    0.564642f, 0.000000f,  0.825336f, 0.038325f,
+    0.000000f, 0.000000f,  0.000000f, 1.000000f;
+
+  // apply some additional, random transformation to show that the initial point cloud poses do not matter
+  const Eigen::Affine3f additional_transformation = Eigen::Translation3f(-0.515f, 0.260f, -0.845f) *
+      Eigen::AngleAxisf(-1.627f, Eigen::Vector3f(0.354f, 0.878f, -0.806f).normalized());
   PointCloud<PointXYZ> cloud_source_transformed;
-  transformPointCloud (cloud_source, cloud_source_transformed, initial_offset, initial_rotation);
+  transformPointCloud (cloud_source, cloud_source_transformed, additional_transformation);
 
 
   // Create shared pointers
@@ -931,7 +812,7 @@ TEST (PCL, PPFRegistration)
   NormalEstimation<PointXYZ, Normal> normal_estimation;
   search::KdTree<PointXYZ>::Ptr search_tree (new search::KdTree<PointXYZ> ());
   normal_estimation.setSearchMethod (search_tree);
-  normal_estimation.setRadiusSearch (0.05);
+  normal_estimation.setKSearch(30); // nearest-k-search seems to work better than radius-search
   PointCloud<Normal>::Ptr normals_target (new PointCloud<Normal> ()),
       normals_source_transformed (new PointCloud<Normal> ());
   normal_estimation.setInputCloud (cloud_target_ptr);
@@ -953,41 +834,41 @@ TEST (PCL, PPFRegistration)
 
 
   // Train the source cloud - create the hash map search structure
-  PPFHashMapSearch::Ptr hash_map_search (new PPFHashMapSearch (15.0 / 180 * M_PI,
+  PPFHashMapSearch::Ptr hash_map_search (new PPFHashMapSearch (2.0 * M_PI / 36, // divide into 36 steps
                                                                0.05));
   hash_map_search->setInputFeatureCloud (features_source_transformed);
 
   // Finally, do the registration
   PPFRegistration<PointNormal, PointNormal> ppf_registration;
-  ppf_registration.setSceneReferencePointSamplingRate (20);
+  ppf_registration.setSceneReferencePointSamplingRate (5);
   ppf_registration.setPositionClusteringThreshold (0.15);
-  ppf_registration.setRotationClusteringThreshold (45.0 / 180 * M_PI);
+  ppf_registration.setRotationClusteringThreshold (25.0 / 180 * M_PI);
   ppf_registration.setSearchMethod (hash_map_search);
-  ppf_registration.setInputCloud (cloud_source_transformed_with_normals);
+  ppf_registration.setInputSource (cloud_source_transformed_with_normals);
   ppf_registration.setInputTarget (cloud_target_with_normals);
 
   PointCloud<PointNormal> cloud_output;
   ppf_registration.align (cloud_output);
   Eigen::Matrix4f transformation = ppf_registration.getFinalTransformation ();
 
-  EXPECT_NEAR (transformation(0, 0), -0.153768, 1e-4);
-  EXPECT_NEAR (transformation(0, 1), -0.628136, 1e-4);
-  EXPECT_NEAR (transformation(0, 2), 0.762759, 1e-4);
-  EXPECT_NEAR (transformation(0, 3), 15.472, 1e-4);
-  EXPECT_NEAR (transformation(1, 0), 0.967397, 1e-4);
-  EXPECT_NEAR (transformation(1, 1), -0.252918, 1e-4);
-  EXPECT_NEAR (transformation(1, 2), -0.0132578, 1e-4);
-  EXPECT_NEAR (transformation(1, 3), -96.6221, 1e-4);
-  EXPECT_NEAR (transformation(2, 0), 0.201243, 1e-4);
-  EXPECT_NEAR (transformation(2, 1), 0.735852, 1e-4);
-  EXPECT_NEAR (transformation(2, 2), 0.646547, 1e-4);
-  EXPECT_NEAR (transformation(2, 3), -20.134, 1e-4);
-  EXPECT_NEAR (transformation(3, 0), 0.000000, 1e-4);
-  EXPECT_NEAR (transformation(3, 1), 0.000000, 1e-4);
-  EXPECT_NEAR (transformation(3, 2), 0.000000, 1e-4);
-  EXPECT_NEAR (transformation(3, 3), 1.000000, 1e-4);
+  const Eigen::Matrix4f reference_transformation = bun0_to_bun4_groundtruth * additional_transformation.inverse().matrix();
+  EXPECT_NEAR (transformation(0, 0), reference_transformation(0, 0), 0.1);
+  EXPECT_NEAR (transformation(0, 1), reference_transformation(0, 1), 0.1);
+  EXPECT_NEAR (transformation(0, 2), reference_transformation(0, 2), 0.1);
+  EXPECT_NEAR (transformation(0, 3), reference_transformation(0, 3), 0.1);
+  EXPECT_NEAR (transformation(1, 0), reference_transformation(1, 0), 0.1);
+  EXPECT_NEAR (transformation(1, 1), reference_transformation(1, 1), 0.1);
+  EXPECT_NEAR (transformation(1, 2), reference_transformation(1, 2), 0.1);
+  EXPECT_NEAR (transformation(1, 3), reference_transformation(1, 3), 0.1);
+  EXPECT_NEAR (transformation(2, 0), reference_transformation(2, 0), 0.1);
+  EXPECT_NEAR (transformation(2, 1), reference_transformation(2, 1), 0.1);
+  EXPECT_NEAR (transformation(2, 2), reference_transformation(2, 2), 0.1);
+  EXPECT_NEAR (transformation(2, 3), reference_transformation(2, 3), 0.1);
+  EXPECT_NEAR (transformation(3, 0), 0.0f, 1e-6);
+  EXPECT_NEAR (transformation(3, 1), 0.0f, 1e-6);
+  EXPECT_NEAR (transformation(3, 2), 0.0f, 1e-6);
+  EXPECT_NEAR (transformation(3, 3), 1.0f, 1e-6);
 }
-#endif
 
 /* ---[ */
 int
@@ -1019,10 +900,10 @@ main (int argc, char** argv)
   testing::InitGoogleTest (&argc, argv);
   return (RUN_ALL_TESTS ());
 
-  // Tranpose the cloud_model
-  /*for (size_t i = 0; i < cloud_model.points.size (); ++i)
+  // Transpose the cloud_model
+  /*for (std::size_t i = 0; i < cloud_model.size (); ++i)
   {
-  //  cloud_model.points[i].z += 1;
+  //  cloud_model[i].z += 1;
   }*/
 }
 /* ]--- */
